@@ -1,180 +1,76 @@
-/* -*- Mode: Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: buildhash.c,v 1.44 1992/01/07 10:04:46 geoff Exp $";
+    "$Id: buildhash.c,v 1.60 1994/01/25 07:11:18 geoff Exp $";
 #endif
 
 #define MAIN
 
 /*
- * buildhash.c - make a hash table for ispell
+ * buildhash.c - make a hash table for okspell
  *
  * Pace Willisson, 1983
  *
- * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
+ * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 /*
  * $Log: buildhash.c,v $
- * Revision 1.44  1992/01/07  10:04:46  geoff
- * Fix an unintentional infinite loop in the overflow processing.
+ * Revision 1.60  1994/01/25  07:11:18  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
- * Revision 1.43  91/07/15  19:26:37  geoff
- * Provide the "canonical" parameter to all strtoichar and strtosichar calls.
- * 
- * Revision 1.42  91/07/11  19:51:53  geoff
- * Move the include files so that param.h gets included before stat.h.
- * Remove the include of stdio.h, since that's now done by ispell.h.
- * 
- * Revision 1.41  91/07/05  20:31:49  geoff
- * Fix some more lint complaints.
- * 
- * Revision 1.40  91/07/05  19:51:35  geoff
- * Fix a bunch of lint complaints.
- * 
- * Revision 1.39  91/07/03  18:20:13  geoff
- * Don't include types.h, dir.h, or param.h, since config.h now does that.
- * 
- * Revision 1.38  91/06/23  22:01:00  geoff
- * On non-USG systems, include sys/dir.h for MAXNAMLEN
- * 
- * Revision 1.37  91/05/27  21:32:54  geoff
- * Add code to write the string character type tables to the hash file.
- * 
- * Revision 1.36  91/01/27  00:43:29  geoff
- * Replace the old hard-to-interpret statistics report with a new one which
- * directly gives the number of probes, number of words taking that number
- * of probes, cumulative total number of words, and cumulative average
- * number of probes.
- * 
- * Revision 1.35  90/12/31  00:58:51  geoff
- * Reformat to follow a consistent convention throughout ispell
- * 
- * Revision 1.34  89/12/27  03:16:50  geoff
- * Move all messages to msgs.h so they can be reconfigured
- * 
- * Revision 1.33  89/10/20  00:11:04  geoff
- * Add version.h
- * 
- * Revision 1.32  89/07/11  00:18:38  geoff
- * Add Amiga support from luis@rice.edu.
- * 
- * Revision 1.31  89/06/09  15:51:32  geoff
- * Add support for the internal "character" type, ichar_t.
- * 
- * Revision 1.30  89/04/28  01:06:34  geoff
- * Change Header to Id;  nobody cares about my pathnames.
- * 
- * Revision 1.29  89/04/03  00:59:34  geoff
- * Fix a bunch of lint complaints (mostly casting unused return values to
- * void).  Also add support for the selectable flag marker.
- * 
- * Revision 1.28  89/02/20  22:09:34  geoff
- * Move a misplaced endif so we don't get into infinite loops if
- * capitalization is turned off.
- * 
- * Revision 1.27  89/01/30  22:54:55  geoff
- * Fix a misplaced right curly brace (Ken Stevens)
- * 
- * Revision 1.26  88/12/26  02:22:41  geoff
- * Add a copyright notice.
- * 
- * Revision 1.25  88/04/30  22:11:56  geoff
- * Fix some lint complaints.
- * 
- * Revision 1.24  88/03/27  00:56:48  geoff
- * Call combinecaps even when compiled without CAPITALIZATION.
- * 
- * Revision 1.23  88/03/23  00:40:43  geoff
- * Add the -s (silent) option.  Fix the reporting of table overflows
- * to be non-interactive, and to give only one message with the count.
- * Add a missing ifdef on CAPITALIZATION.  Get rid of an unused variable.
- * 
- * Revision 1.22  88/03/12  02:42:52  geoff
- * Correctly check for hash table overflow (>=, not >).  Also make the test
- * slightly more efficient by avoiding a multiply and add every loop trip.
- * Make sure the bumping of freepointer doesn't over-read the hash table.
- * 
- * Revision 1.21  88/02/20  23:09:35  geoff
- * Add the "-c" option to replace icombine.  Major changes to use the new
- * capitalization format.
- * 
- * Revision 1.20  87/09/26  15:52:35  geoff
- * Add code to check the maximum affix extension in the language table
- * and print a warning if there's a problem.
- * 
- * Revision 1.19  87/09/09  00:15:20  geoff
- * Speed up by a factor of five by calling malloc much less, and for larger
- * chunks of memory.  (This is somewhat, though not terribly, wasteful of
- * memory).
- * 
- * Revision 1.18  87/08/28  21:19:50  geoff
- * Change the argument order so that the hash file is last again.
- * 
- * Revision 1.17  87/07/20  23:20:31  geoff
- * Add code to parse and write out the language tables.  Delete the old
- * English-specific flag code.
- * 
- * Revision 1.16  87/06/09  00:15:21  geoff
- * Add a missing #else clause for CAPITALIZE
- * 
- * Revision 1.15  87/06/07  16:10:02  geoff
- * Fix a syntax error in the non-CAPITALIZE mode
- * 
- * Revision 1.14  87/05/30  13:00:55  geoff
- * Stefan Taxhet's missing-argument bugfix
- * 
- * Revision 1.13  87/04/24  20:31:54  geoff
- * Recognize slash as a word terminator in makedent.
- * 
- * Revision 1.12  87/04/21  23:28:49  geoff
- * Pad to a struct dent boundary with null bytes, not with near-garbage.
- * 
- * Revision 1.11  87/04/19  22:50:44  geoff
- * Add extensive code to handle capitalization.
- * 
- * Revision 1.10  87/04/01  15:21:53  geoff
- * Integrate Joe Orost's V7/register changes into the main branch
- * 
- * Revision 1.9  87/03/30  00:33:43  geoff
- * Move the include of stat.h so it compiles on BSD systems (Ken Yap).
- * 
- * Revision 1.8  87/03/27  17:20:22  geoff
- * Accept (but don't require) new-format dictionaries without excess slashes.
- * 
- * Revision 1.7  87/03/26  00:40:38  geoff
- * Don't include types.h on BSD systems (Jim Knutson)
- * 
- * Revision 1.6  87/03/26  00:30:23  geoff
- * Integrate Rich Salz's changes/improvements
- * 
- * Revision 1.5  87/03/23  00:07:27  geoff
- * Don Kark's don't-save-the-whole-dictionary bug fix
- * 
- * Revision 1.4  87/02/26  00:41:52  geoff
- * Integrate McQueer's and Ries's enhancements/fixes into the main branch
- * 
- * Revision 1.3  87/01/20  14:17:04  geoff
- * Return an explicit exit status
- * 
- * Revision 1.2  87/01/17  13:11:22  geoff
- * Add RCS ID keywords
- * 
  */
 
 #include "config.h"
 #include "ispell.h"
+#include "proto.h"
 #include "msgs.h"
 #include "version.h"
 #include <ctype.h>
-#ifdef AMIGA
-#include <stat.h>
-#else /* AMIGA */
 #include <sys/stat.h>
-#endif /* AMIGA */
+
+int		main P ((int argc, char * argv[]));
+static void	output P ((void));
+static void	filltable P ((void));
+VOID *		mymalloc P ((unsigned int size));
+VOID *		myrealloc P ((VOID * ptr, unsigned int size,
+		  unsigned int oldsize));
+void		myfree P ((VOID * ptr));
+static void	readdict P ((void));
+static void	newcount P ((void));
 
 #define NSTAT	100		/* Size probe-statistics table */
 
@@ -183,19 +79,6 @@ struct stat	cstat;		/* Result of stat-ing count file */
 
 int		hashsize;	/* Number of entries to go in hash table */
 
-extern char *	index ();
-extern char *	calloc ();
-extern char *	malloc ();
-extern char *	realloc ();
-extern char *	strcpy ();
-extern void	exit ();
-
-extern void	yyinit ();
-extern void	upcase ();
-#ifdef CAPITALIZATION
-extern long	whatcap ();
-#endif
-
 struct hashheader hashheader;	/* Header of hash table being built */
 struct dent *	hashtbl;	/* Entries to go in hash table */
 
@@ -208,19 +91,16 @@ char		Sfile[MAXPATHLEN]; /* Name of statistics file */
 
 static int silent = 0;		/* NZ to suppress count reports */
 
-main (argc, argv)
+int main (argc, argv)
     int		argc;
     char *	argv[];
     {
+    int		avg;
     FILE *	countf;
     FILE *	statf;
     int		stats[NSTAT];
     int		i;
-    int		combine = 0;
-
-#ifdef AMIGA
-    _Heapsize = 2*HEAPSIZE;
-#endif /* AMIGA */
+    int		j;
 
     while (argc > 1  &&  *argv[1] == '-')
 	{
@@ -228,56 +108,21 @@ main (argc, argv)
 	argv++;
 	switch (argv[0][1])
 	    {
-	    case 'c':
-		if (argc > 1)
-		    {
-		    hashsize = atoi (argv[1]);
-		    argc--;
-		    argv++;
-		    }
-		combine = 1;
-		break;
 	    case 's':
 		silent = 1;
 		break;
 	    }
 	}
-    if (combine)
+    if (argc == 4)
 	{
-	if (argc > 1)
-	    Lfile = argv[1];
-	else
-	    Lfile = DEFLANG;
-	}
-    else if (argc > 1)
-	{
-	++argv;
-	Dfile = *argv;
-	if (argc > 2)
-	    {
-	    ++argv;
-	    Lfile = *argv;
-	    if (argc > 3)
-		{
-		++argv;
-		Hfile = *argv;
-		}
-	    else
-		{
-		Hfile = DEFHASH;
-		}
-	    }
-	else
-	    {
-	    Lfile = DEFLANG;
-	    Hfile = DEFHASH;
-	    }
+	Dfile = argv[1];
+	Lfile = argv[2];
+	Hfile = argv[3];
 	}
     else
 	{
-	Dfile = DEFDICT;
-	Lfile = DEFLANG;
-	Hfile = DEFHASH;
+	(void) fprintf (stderr, BHASH_C_USAGE);
+	return 1;
 	}
 
     if (yyopen (Lfile))			/* Open the language file */
@@ -286,110 +131,75 @@ main (argc, argv)
     if (yyparse ())			/* Parse the language tables */
 	exit (1);
 
-    if (!combine)
+    (void) sprintf (Cfile, "%s.cnt", Dfile);
+    (void) sprintf (Sfile, "%s.stat", Dfile);
+
+    if (stat (Dfile, &dstat) < 0)
 	{
-	(void) sprintf (Cfile, "%s.cnt", Dfile);
-	(void) sprintf (Sfile, "%s.stat", Dfile);
-
-	if (stat (Dfile, &dstat) < 0)
-	    {
-	    (void) fprintf (stderr, BHASH_C_NO_DICT, Dfile);
-		exit (1);
-	    }
-	if (stat (Cfile, &cstat) < 0 || dstat.st_mtime > cstat.st_mtime)
-	    newcount ();
-
-	if ((countf = fopen (Cfile, "r")) == NULL)
-	    {
-	    (void) fprintf (stderr, BHASH_C_NO_COUNT);
+	(void) fprintf (stderr, BHASH_C_NO_DICT, Dfile);
 	    exit (1);
-	    }
-	hashsize = 0;
-	(void) fscanf (countf, "%d", &hashsize);
-	(void) fclose (countf);
-	if (hashsize == 0)
-	    {
-	    (void) fprintf (stderr, BHASH_C_BAD_COUNT);
-	    exit (1);
-	    }
 	}
-    readdict (combine);
+    if (stat (Cfile, &cstat) < 0 || dstat.st_mtime > cstat.st_mtime)
+	newcount ();
 
-    if (combine)
-	combineout ();
-    else
+    if ((countf = fopen (Cfile, "r")) == NULL)
 	{
-	int		    avg;
-	int		    j;
-
-	if ((statf = fopen (Sfile, "w")) == NULL)
-	    {
-	    (void) fprintf (stderr, CANT_CREATE, Sfile);
-	    exit (1);
-	    }
-
-	for (i = 0; i < NSTAT; i++)
-	    stats[i] = 0;
-	for (i = 0; i < hashsize; i++)
-	    {
-	    struct dent *   dp;
-
-	    dp = &hashtbl[i];
-	    if ((dp->flagfield & USED) != 0)
-		{
-		for (j = 0;  dp != NULL;  j++, dp = dp->next)
-		    {
-		    if (j >= NSTAT)
-			j = NSTAT - 1;
-		    stats[j]++;
-		    }
-		}
-	    }
-	for (i = 0, j = 0, avg = 0;  i < NSTAT;  i++)
-	    {
-	    j += stats[i];
-	    avg += stats[i] * (i + 1);
-	    if (j == 0)
-		(void) fprintf (statf, "%d:\t%d\t0\t0.0\n", i + 1, stats[i]);
-	    else
-		(void) fprintf (statf, "%d:\t%d\t%d\t%f\n", i + 1, stats[i], j,
-		  (double) avg / j);
-	    }
-	(void) fclose (statf);
-
-	filltable ();
-	output ();
+	(void) fprintf (stderr, BHASH_C_NO_COUNT);
+	exit (1);
 	}
-    exit(0);
-    /* NOTREACHED */
-    }
-
-combineout ()
-    {
-    register struct dent *  dp;
-    register struct dent *  tdp;
-    register int	    i;
-
-    /* Put out the dictionary on stdout in text format */
-    for (i = 0, dp = hashtbl;  i < hashsize;  i++, dp++)
+    hashsize = 0;
+    (void) fscanf (countf, "%d", &hashsize);
+    (void) fclose (countf);
+    if (hashsize == 0)
 	{
-	if (dp->flagfield & USED)
+	(void) fprintf (stderr, BHASH_C_BAD_COUNT);
+	exit (1);
+	}
+    readdict ();
+
+    if ((statf = fopen (Sfile, "w")) == NULL)
+	{
+	(void) fprintf (stderr, CANT_CREATE, Sfile);
+	exit (1);
+	}
+
+    for (i = 0; i < NSTAT; i++)
+	stats[i] = 0;
+    for (i = 0; i < hashsize; i++)
+	{
+	struct dent *   dp;
+
+	dp = &hashtbl[i];
+	if ((dp->flagfield & USED) != 0)
 	    {
-	    for (tdp = dp;  tdp != NULL;  tdp = tdp->next)
+	    for (j = 0;  dp != NULL;  j++, dp = dp->next)
 		{
-		toutent (stdout, tdp, 0);
-#ifdef CAPITALIZATION
-		while (tdp->flagfield & MOREVARIANTS)
-		    tdp = tdp->next;
-#endif
+		if (j >= NSTAT)
+		    j = NSTAT - 1;
+		stats[j]++;
 		}
 	    }
 	}
+    for (i = 0, j = 0, avg = 0;  i < NSTAT;  i++)
+	{
+	j += stats[i];
+	avg += stats[i] * (i + 1);
+	if (j == 0)
+	    (void) fprintf (statf, "%d:\t%d\t0\t0.0\n", i + 1, stats[i]);
+	else
+	    (void) fprintf (statf, "%d:\t%d\t%d\t%f\n", i + 1, stats[i], j,
+	      (double) avg / j);
+	}
+    (void) fclose (statf);
+
+    filltable ();
+    output ();
+    return 0;
     }
 
-output ()
+static void output ()
     {
-    register FILE *		outfile;
+    register FILE *		houtfile;
     register struct dent *	dp;
     int				strptr;
     int				n;
@@ -398,7 +208,7 @@ output ()
     int				maxslen;
     struct flagent *		fentry;
 
-    if ((outfile = fopen (Hfile, "w")) == NULL)
+    if ((houtfile = fopen (Hfile, "wb")) == NULL)
 	{
 	(void) fprintf (stderr, CANT_CREATE, Hfile);
 	return;
@@ -406,7 +216,7 @@ output ()
     hashheader.stringsize = 0;
     hashheader.lstringsize = 0;
     hashheader.tblsize = hashsize;
-    (void) fwrite ((char *) &hashheader, sizeof hashheader, 1, outfile);
+    (void) fwrite ((char *) &hashheader, sizeof hashheader, 1, houtfile);
     strptr = 0;
     /*
     ** Put out the strings from the flags table.  This code assumes that
@@ -420,14 +230,14 @@ output ()
 	if (fentry->stripl)
 	    {
 	    (void) fwrite ((char *) fentry->strip, fentry->stripl + 1,
-	      sizeof (ichar_t), outfile);
+	      sizeof (ichar_t), houtfile);
 	    fentry->strip = (ichar_t *) strptr;
 	    strptr += (fentry->stripl + 1) * sizeof (ichar_t);
 	    }
 	if (fentry->affl)
 	    {
 	    (void) fwrite ((char *) fentry->affix, fentry->affl + 1,
-	      sizeof (ichar_t), outfile);
+	      sizeof (ichar_t), houtfile);
 	    fentry->affix = (ichar_t *) strptr;
 	    strptr += (fentry->affl + 1) * sizeof (ichar_t);
 	    }
@@ -443,14 +253,14 @@ output ()
 	if (fentry->stripl)
 	    {
 	    (void) fwrite ((char *) fentry->strip, fentry->stripl + 1,
-	      sizeof (ichar_t), outfile);
+	      sizeof (ichar_t), houtfile);
 	    fentry->strip = (ichar_t *) strptr;
 	    strptr += (fentry->stripl + 1) * sizeof (ichar_t);
 	    }
 	if (fentry->affl)
 	    {
 	    (void) fwrite ((char *) fentry->affix, fentry->affl + 1,
-	      sizeof (ichar_t), outfile);
+	      sizeof (ichar_t), houtfile);
 	    fentry->affix = (ichar_t *) strptr;
 	    strptr += (fentry->affl + 1) * sizeof (ichar_t);
 	    }
@@ -467,14 +277,17 @@ output ()
     for (i = 0;  i < hashheader.nstrchartype;  i++)
 	{
 	n = strlen (chartypes[i].name) + 1;
-	(void) fwrite (chartypes[i].name, n, 1, outfile);
+	(void) fwrite (chartypes[i].name, n, 1, houtfile);
+	strptr += n;
+	n = strlen (chartypes[i].deformatter) + 1;
+	(void) fwrite (chartypes[i].deformatter, n, 1, houtfile);
 	strptr += n;
 	for (n = 0;
 	  chartypes[i].suffixes[n] != '\0';
 	  n += strlen (&chartypes[i].suffixes[n]) + 1)
 	    ;
 	n++;
-	(void) fwrite (chartypes[i].suffixes, n, 1, outfile);
+	(void) fwrite (chartypes[i].suffixes, n, 1, houtfile);
 	strptr += n;
 	}
     hashheader.lstringsize = strptr;
@@ -494,7 +307,7 @@ output ()
 	else
 	    {
 	    n = strlen (dp->word) + 1;
-	    (void) fwrite (dp->word, n, 1, outfile);
+	    (void) fwrite (dp->word, n, 1, houtfile);
 	    dp->word = (char *) strptr;
 	    strptr += n;
 	    }
@@ -506,7 +319,7 @@ output ()
 	n = sizeof (struct dent) - n;
 	strptr += n;
 	while (--n >= 0)
-	    (void) putc ('\0', outfile);
+	    (void) putc ('\0', houtfile);
 	}
     /* Put out the hash table itself */
     for (i = 0, dp = hashtbl;  i < hashsize;  i++, dp++)
@@ -521,23 +334,28 @@ output ()
 	    {
 	    dp->next = (struct dent *)-1;
 	    }
+#ifdef PIECEMEAL_HASH_WRITES
+	(void) fwrite ((char *) dp, sizeof (struct dent), 1, houtfile);
+#endif /* PIECEMEAL_HASH_WRITES */
 	}
-    (void) fwrite ((char *) hashtbl, sizeof (struct dent), hashsize, outfile);
+#ifndef PIECEMEAL_HASH_WRITES
+    (void) fwrite ((char *) hashtbl, sizeof (struct dent), hashsize, houtfile);
+#endif /* PIECEMEAL_HASH_WRITES */
     /* Put out the language tables */
     (void) fwrite ((char *) sflaglist,
-      sizeof (struct flagent), numsflags, outfile);
+      sizeof (struct flagent), numsflags, houtfile);
     hashheader.stblsize = numsflags;
     (void) fwrite ((char *) pflaglist,
-      sizeof (struct flagent), numpflags, outfile);
+      sizeof (struct flagent), numpflags, houtfile);
     hashheader.ptblsize = numpflags;
     /* Finish filling in the hash header. */
     hashheader.stringsize = strptr;
-    rewind (outfile);
-    (void) fwrite ((char *) &hashheader, sizeof hashheader, 1, outfile);
-    (void) fclose (outfile);
+    rewind (houtfile);
+    (void) fwrite ((char *) &hashheader, sizeof hashheader, 1, houtfile);
+    (void) fclose (houtfile);
     }
 
-filltable ()
+static void filltable ()
     {
     struct dent *freepointer, *nextword, *dp;
     struct dent *hashend;
@@ -581,25 +399,25 @@ filltable ()
     }
 
 #if MALLOC_INCREMENT == 0
-char * mymalloc (size)
-    int		size;
+VOID * mymalloc (size)
+    unsigned int	size;
     {
 
     return malloc (size);
     }
 
 /* ARGSUSED */
-char * myrealloc (ptr, size, oldsize)
-    char *	ptr;
-    int		size;
-    int		oldsize;
+VOID * myrealloc (ptr, size, oldsize)
+    VOID *		ptr;
+    unsigned int	size;
+    unsigned int	oldsize;
     {
 
     return realloc (ptr, size);
     }
 
 void myfree (ptr)
-    char *	ptr;
+    VOID *	ptr;
     {
 
     free (ptr);
@@ -607,12 +425,12 @@ void myfree (ptr)
 
 #else
 
-char * mymalloc (size)		/* Fast, unfree-able variant of malloc */
-    int			size;
+VOID * mymalloc (size)		/* Fast, unfree-able variant of malloc */
+    unsigned int	size;
     {
-    char *		retval;
+    VOID *		retval;
     static int		bytesleft = 0;
-    static char *	nextspace;
+    static VOID *	nextspace;
 
     if (size < 4)
 	size = 4;
@@ -628,17 +446,17 @@ char * mymalloc (size)		/* Fast, unfree-able variant of malloc */
 	    }
 	}
     retval = nextspace;
-    nextspace += size;
+    nextspace = (VOID *) ((char *) nextspace + size);
     bytesleft -= size;
     return retval;
     }
 
-char * myrealloc (ptr, size, oldsize)
-    char *		ptr;
-    int			size;
-    int			oldsize;
+VOID * myrealloc (ptr, size, oldsize)
+    VOID *		ptr;
+    unsigned int	size;
+    unsigned int	oldsize;
     {
-    char *nptr;
+    VOID *nptr;
 
     nptr = mymalloc (size);
     if (nptr == NULL)
@@ -649,13 +467,12 @@ char * myrealloc (ptr, size, oldsize)
 
 /* ARGSUSED */
 void myfree (ptr)
-    char *		ptr;
+    VOID *		ptr;
     {
     }
 #endif
 
-readdict (combine)
-    int			combine;
+static void readdict ()
     {
     struct dent		d;
     register struct dent * dp;
@@ -666,15 +483,10 @@ readdict (combine)
     int			i;
     int			h;
 
-    if (combine)
-	dictf = stdin;
-    else
+    if ((dictf = fopen (Dfile, "r")) == NULL)
 	{
-	if ((dictf = fopen (Dfile, "r")) == NULL)
-	    {
-	    (void) fprintf (stderr, BHASH_C_CANT_OPEN_DICT);
-	    exit (1);
-	    }
+	(void) fprintf (stderr, BHASH_C_CANT_OPEN_DICT);
+	exit (1);
 	}
 
     hashtbl =
@@ -695,7 +507,7 @@ readdict (combine)
 	    }
 	i++;
 
-	if (makedent (lbuf, &d) < 0)
+	if (makedent (lbuf, sizeof lbuf, &d) < 0)
 	    continue;
 
 	h = hash (strtosichar (d.word, 1), hashsize);
@@ -704,7 +516,7 @@ readdict (combine)
 	if ((dp->flagfield & USED) == 0)
 	    {
 	    *dp = d;
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 	    /*
 	    ** If it's a followcase word, we need to make this a
 	    ** special dummy entry, and add a second with the
@@ -732,10 +544,10 @@ readdict (combine)
 		{
 		if (strcmp (dp->word, ucbuf) == 0)
 		    break;
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 		while (dp->flagfield & MOREVARIANTS)
 		    dp = dp->next;
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
 		dp = dp->next;
 		}
 	    if (dp != NULL)
@@ -762,7 +574,7 @@ readdict (combine)
 		*dp = d;
 		lastdp->next = dp;
 		dp->next = NULL;
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 		/*
 		** If it's a followcase word, we need to make this a
 		** special dummy entry, and add a second with the
@@ -779,19 +591,18 @@ readdict (combine)
 	}
     if (!silent)
 	(void) fprintf (stderr, "\n");
-    if (!combine)
-	(void) fclose (dictf);
+    (void) fclose (dictf);
     }
 
-newcount ()
+static void newcount ()
     {
     char		buf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
     ichar_t		ibuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
 #endif
     register FILE *	d;
     register int	i;
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
     ichar_t		lastibuf[sizeof ibuf / sizeof (ichar_t)];
     int			headercounted;
     int			followcase;
@@ -807,7 +618,7 @@ newcount ()
 	exit (1);
 	}
 
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
     headercounted = 0;
     lastibuf[0] = 0;
 #endif
@@ -818,11 +629,12 @@ newcount ()
 	    (void) fprintf (stderr, "%d ", i);
 	    (void) fflush (stdout);
 	    }
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 	cp = index (buf, hashheader.flagmarker);
 	if (cp != NULL)
 	    *cp = '\0';
-	strtoichar (ibuf, buf, 1);
+	if (strtoichar (ibuf, buf, INPUTWORDLEN * sizeof (ichar_t), 1))
+	    (void) fprintf (stderr, WORD_TOO_LONG (buf));
 	followcase = (whatcap (ibuf) == FOLLOWCASE);
 	upcase (ibuf);
 	if (icharcmp (ibuf, lastibuf) != 0)
