@@ -1,7 +1,6 @@
-/* -*- Mode:Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: defmt.c,v 1.15 1992/01/07 10:04:51 geoff Exp $";
+    "$Id: defmt.c,v 1.31 1994/05/25 04:29:28 geoff Exp $";
 #endif
 
 /*
@@ -12,10 +11,41 @@ static char Rcs_Id[] =
  *
  * Copyright (c), 1983, by Pace Willisson
  *
- * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
+ * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  *
  * The TeX code is originally by Greg Schaffer, with many improvements from
  * Ken Stevens.  The nroff code is primarily from Pace Willisson, although
@@ -24,105 +54,48 @@ static char Rcs_Id[] =
 
 /*
  * $Log: defmt.c,v $
- * Revision 1.15  1992/01/07  10:04:51  geoff
- * Be a bit smarter about only putting out linefeeds if they were there
- * in the inpu file; this is necessary to correctly handle very long
- * input lines.
+ * Revision 1.31  1994/05/25  04:29:28  geoff
+ * If two boundary characters appear in a row, consider it the end of the
+ * word.
  *
- * Revision 1.14  1992/01/04  22:08:11  geoff
- * Correctly handle TeX display-math mode (doubled dollar signs).  Add
- * some comments to the code that handles tib(1) bibliography entries.
- * Fix the tib(1) handling so that it doesn't break if the (apparent)
- * reference crosses a line boundary.
+ * Revision 1.30  1994/05/17  06:44:08  geoff
+ * Add the new argument to all calls to good and compoundgood.
  *
- * Revision 1.13  1991/12/09  00:40:35  geoff
- * Get rid of a never-reached return statement.
+ * Revision 1.29  1994/03/16  06:30:41  geoff
+ * Don't lose track of math mode when an array environment is embedded.
  *
- * Revision 1.12  91/09/12  00:01:30  geoff
- * Many changes to improve TeX deformatting: Recognize comments, so that
- * things like dollar signs in comments don't end math mode.  Recognize
- * the "tib" bibliography citation syntax.  Fix a number of places where
- * the parsing got a bit confused.
- * 
- * Revision 1.11  91/07/15  19:26:47  geoff
- * Provide the "canonical" parameter to all *isstringch, strtoichar, and
- * strtosichar calls.
- * 
- * Revision 1.10  91/07/11  19:51:59  geoff
- * Remove the include of stdio.h, since ispell.h now does this.
- * 
- * Revision 1.9  91/07/03  18:20:27  geoff
- * Replace all conversions of chars to ichar_t's with a macro call which
- * (a) does the conversion correctly and (b) makes future changes easy in
- * case (a) is false.
- * 
- * Revision 1.8  91/06/23  22:08:54  geoff
- * When casting to ichar_t, cast to unsigned first to avoid sign-extension
- * problems.
- * 
- * Revision 1.7  90/12/31  00:59:04  geoff
- * Reformat to follow a consistent convention throughout ispell
- * 
- * Revision 1.6  90/10/05  01:54:04  geoff
- * Kenny Stevens' latest fixes:  minword didn't work in ask mode;  LaTeX
- * could put you in math mode unexpectedly, and a couple of other small
- * (he says) bugs.
- * 
- * Revision 1.5  90/04/17  15:31:14  geoff
- * Fix the TeX parsing routines to accept a buffer pointer to be updated,
- * rather than using the global "currentchar."  This fixes a bug in TeX
- * parsing where the routines looked at the wrong character in the line.
- * 
- * Revision 1.4  89/12/27  22:25:40  geoff
- * Add minword support.
- * 
- * Revision 1.3  89/12/27  03:17:50  geoff
- * Move all messages to msgs.h so they can be reconfigured
- * 
- * Revision 1.2  89/10/20  00:10:25  geoff
- * Remove version.h
- * 
- * Revision 1.1  89/06/27  01:56:02  geoff
- * Break ispell.c up into smaller files
- * 
+ * Revision 1.28  1994/03/15  05:31:57  geoff
+ * Add TeX_strncmp, which allows us to handle AMS-TeX constructs like
+ * \endroster without getting confused.
  *
- * The following log lines are from when this code was in ispell.c:
+ * Revision 1.27  1994/02/14  00:34:53  geoff
+ * Pass length arguments to correct().
+ *
+ * Revision 1.26  1994/01/25  07:11:25  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
- * Revision 1.63  89/02/17  14:10:31  geoff
- * Improve the DeTex processing some more (Ken Stevens).
- * 
- * Revision 1.52  88/06/25  17:48:26  geoff
- * Add support for the new switches -n, -b, -B, -C, -P, and -m, for the
- * keeping of TeX/nroff special characters in the hash header, and for
- * setting certain defaults in the hash header.  Note that the -B/-C switches
- * add the new routine "compoundgood", and the -P/-m switches replace
- * "nopossibilities" with "easypossibilities" and change the emacs output
- * format.
- * 
- * Revision 1.34  87/06/07  15:25:09  geoff
- * At Don Kark's suggestion, improve ISTEXTERM
- * 
- * Revision 1.33  87/06/07  15:15:24  geoff
- * Integrate Don Karks's TeX brace-handling
- * 
- * Revision 1.18  87/03/31  16:40:59  geoff
- * Integrate Steve Kelem's changes into the main branch:  really wait for
- * a space in givehelp();  accept +/- in troff size strings, and automatically
- * set TeX mode if the file extension is ".tex".
- * 
- * Revision 1.17  87/03/31  15:32:55  geoff
- * Improve troff backslash handling some more (isaac@mulga.oz)
- * 
- * Revision 1.11  87/03/23  00:03:48  geoff
- * Let's try again:  Greg Schaffer's fixed TeX stuff
- * 
  */
 
 #include <ctype.h>
 #include "config.h"
 #include "ispell.h"
+#include "proto.h"
 #include "msgs.h"
 
+static char *	skiptoword P ((char * bufp));
+char *		skipoverword P ((char * bufp));
+void		checkline P ((FILE * ofile));
+static int	TeX_math_end P ((char ** bufp));
+static int	TeX_math_begin P ((char ** bufp));
+static int	TeX_LR_begin P ((char ** bufp));
+static int	TeX_LR_check P ((int begin_p, char ** bufp));
+static void	TeX_skip_args P ((char ** bufp));
+static int	TeX_math_check P ((int cont_char, char ** bufp));
+static void	TeX_skip_parens P ((char ** bufp));
+static void	TeX_open_paren P ((char ** bufp));
+static void	TeX_skip_check P ((char ** bufp));
+static int	TeX_strncmp P ((char * a, char * b, int n));
+
 #define ISTEXTERM(c)   (((c) == TEXLEFTCURLY) || \
 			((c) == TEXRIGHTCURLY) || \
 			((c) == TEXLEFTSQUARE) || \
@@ -131,7 +104,7 @@ static char Rcs_Id[] =
 			((c) == TEXDOLLAR) || \
 			((c) == TEXPERCENT))
 
-char * skiptoword (bufp)		/* Skip to beginning of a word */
+static char * skiptoword (bufp)		/* Skip to beginning of a word */
     char *	bufp;
     {
 
@@ -181,7 +154,7 @@ char * skiptoword (bufp)		/* Skip to beginning of a word */
 		    math_mode--;    /* re-enter math */
 		else if (LaTeX_Mode == 'm'
 		    || (math_mode && (math_mode >= (math_mode & 127) * 128)
-		  &&  (strncmp(bufp, "\\end", 4)
+		  &&  (TeX_strncmp(bufp, "\\end", 4)
 		    == 0)))
 		    {
 		    if (TeX_LR_check(0, &bufp))
@@ -272,7 +245,7 @@ char * skipoverword (bufp)	/* Return pointer to end of a word */
     register int	scharlen; /* Length of a string character */
 
     lastboundary = NULL;
-    while (1)
+    for (  ;  ;  )
 	{
 	if (*bufp == '\0')
 	    {
@@ -298,6 +271,8 @@ char * skipoverword (bufp)	/* Return pointer to end of a word */
 	    {
 	    if (lastboundary == NULL)
 		lastboundary = bufp;
+	    else if (lastboundary == bufp - 1)
+		break;			/* Double boundary -- end of word */
 	    bufp++;
 	    }
 	else
@@ -311,7 +286,7 @@ char * skipoverword (bufp)	/* Return pointer to end of a word */
     return (lastboundary != NULL) ? lastboundary : bufp;
     }
 
-checkline (ofile)
+void checkline (ofile)
     FILE *		ofile;
     {
     register char *	p;
@@ -380,7 +355,7 @@ checkline (ofile)
 	    }
 	}
 
-    while (1)
+    for (  ;  ;  )
 	{
 	p = skiptoword (currentchar);
 	if (p != currentchar)
@@ -394,13 +369,15 @@ checkline (ofile)
 	while (currentchar < endp  &&  p < ctoken + sizeof ctoken - 1)
 	    *p++ = *currentchar++;
 	*p = 0;
-	strtoichar (itoken, ctoken, 0);
+	if (strtoichar (itoken, ctoken, INPUTWORDLEN * sizeof (ichar_t), 0))
+	    (void) fprintf (stderr, WORD_TOO_LONG (ctoken));
 	ilen = icharlen (itoken);
 
 	if (lflag)
 	    {
 	    if (ilen > minword
-	      &&  !good (itoken, 0, 0)  &&  !cflag)
+	      &&  !good (itoken, 0, 0, 0, 0)
+	      &&  !cflag  &&  !compoundgood (itoken, 0))
 		(void) fprintf (ofile, "%s\n", ctoken);
 	    }
 	else
@@ -414,7 +391,7 @@ checkline (ofile)
 			(void) fprintf (ofile, "*\n");
 		    continue;
 		    }
-		if (good (itoken, 0, 0))
+		if (good (itoken, 0, 0, 0, 0))
 		    {
 		    if (hits[0].prefix == NULL
 		      &&  hits[0].suffix == NULL)
@@ -430,7 +407,7 @@ checkline (ofile)
 			  hits[0].dictent->word);
 			}
 		    }
-		else if (compoundgood (itoken))
+		else if (compoundgood (itoken, 0))
 		    {
 		    /* compound-word match */
 		    if (!terse)
@@ -450,7 +427,8 @@ checkline (ofile)
 			  easypossibilities ? '&' : '?',
 			  ctoken,
 			  easypossibilities,
-			  (currentchar - contextbufs[0]) - strlen (ctoken));
+			  (int) ((currentchar - contextbufs[0])
+			    - strlen (ctoken)));
 			for (i = 0;  i < MAXPOSSIBLE;  i++)
 			    {
 			    if (possibilities[i][0] == 0)
@@ -467,14 +445,16 @@ checkline (ofile)
 			*/
 			(void) fprintf (ofile, "# %s %d\n",
 			  ctoken,
-			  (currentchar - contextbufs[0]) - strlen (ctoken));
+			  (int) ((currentchar - contextbufs[0])
+			    - strlen (ctoken)));
 			}
 		    }
 		}
 	    else
 		{
 		if (!quit)
-		   correct (ctoken, itoken, &currentchar);
+		   correct (ctoken, sizeof ctoken, itoken, sizeof itoken,
+		     &currentchar);
 		}
 	    }
 	if (!aflag  &&  !lflag)
@@ -486,7 +466,7 @@ checkline (ofile)
    }
 
 /* must check for \begin{mbox} or whatever makes new text region. */
-int TeX_math_end (bufp)
+static int TeX_math_end (bufp)
     char **	bufp;
     {
 
@@ -509,14 +489,14 @@ int TeX_math_end (bufp)
 	return 1;
     if (TeX_LR_begin (bufp))	/* check for switch back to LR mode */
 	return 1;
-    if (strncmp (*bufp, "end", 3) == 0)
+    if (TeX_strncmp (*bufp, "end", 3) == 0)
 	/* find environment that is ending */
 	return TeX_math_check ('e', bufp);
     else
 	return 0;
     }
 
-int TeX_math_begin (bufp)
+static int TeX_math_begin (bufp)
     char **	bufp;
     {
 
@@ -531,7 +511,7 @@ int TeX_math_begin (bufp)
 	(*bufp)++; /* check for null char here? */
 	if (**bufp == TEXLEFTPAREN  ||  **bufp == TEXLEFTSQUARE)
 	    return 1;
-	if (strncmp (*bufp, "begin", 5) == 0)
+	if (TeX_strncmp (*bufp, "begin", 5) == 0)
 	    {
 	    if (TeX_math_check ('b', bufp))
 		return 1;
@@ -568,17 +548,17 @@ int TeX_math_begin (bufp)
 	return 0;
     }
 
-int TeX_LR_begin (bufp)
+static int TeX_LR_begin (bufp)
     char **	bufp;
     {
 
-    if ((strncmp (*bufp, "mbox", 4) == 0)
-      ||  (strncmp (*bufp, "makebox", 7) == 0)
-      ||  (strncmp (*bufp, "fbox", 4) == 0)
-      || (strncmp (*bufp, "framebox", 8) == 0))
+    if ((TeX_strncmp (*bufp, "mbox", 4) == 0)
+      ||  (TeX_strncmp (*bufp, "makebox", 7) == 0)
+      ||  (TeX_strncmp (*bufp, "fbox", 4) == 0)
+      || (TeX_strncmp (*bufp, "framebox", 8) == 0))
 	math_mode += 2;
-    else if ((strncmp(*bufp, "parbox", 6) == 0)
-      || (strncmp(*bufp, "raisebox", 8) == 0))
+    else if ((TeX_strncmp(*bufp, "parbox", 6) == 0)
+      || (TeX_strncmp(*bufp, "raisebox", 8) == 0))
 	{
 	math_mode += 2;
 	TeX_open_paren (bufp);
@@ -587,7 +567,7 @@ int TeX_LR_begin (bufp)
 	else
 	    LaTeX_Mode = 'r'; /* same as reference -- skip {} */
 	}
-    else if (strncmp(*bufp, "begin", 5) == 0)
+    else if (TeX_strncmp(*bufp, "begin", 5) == 0)
 	return TeX_LR_check (1, bufp);	/* minipage */
     else
 	return 0;
@@ -597,7 +577,7 @@ int TeX_LR_begin (bufp)
     return 1;
     }
 
-int TeX_LR_check (begin_p, bufp)
+static int TeX_LR_check (begin_p, bufp)
     int		begin_p;
     char **	bufp;
     {
@@ -640,7 +620,7 @@ int TeX_LR_check (begin_p, bufp)
 /* Skips the begin{ARG}, and optionally up to two {PARAM}{PARAM}'s to
  *  the begin if they are required.  However, Only skips if on this line.
  */
-void TeX_skip_args (bufp)
+static void TeX_skip_args (bufp)
     char **	bufp;
     {
     register int skip_cnt = 0; /* Max of 2. */
@@ -667,8 +647,8 @@ void TeX_skip_args (bufp)
 	TeX_skip_parens (bufp);	/* skip to end of 2nd {PARAM}. */
     }
 
-int TeX_math_check (cont_char, bufp)
-    char	cont_char;
+static int TeX_math_check (cont_char, bufp)
+    int		cont_char;
     char **	bufp;
     {
 
@@ -676,7 +656,7 @@ int TeX_math_check (cont_char, bufp)
     /* Check for end of line, continue later. */
     if (**bufp == 0)
 	{
-	LaTeX_Mode = cont_char;
+	LaTeX_Mode = (char) cont_char;
 	return 0;
 	}
     else
@@ -685,7 +665,6 @@ int TeX_math_check (cont_char, bufp)
     if (strncmp (++(*bufp), "equation", 8) == 0
       ||  strncmp (*bufp, "eqnarray", 8) == 0
       ||  strncmp (*bufp, "displaymath", 11) == 0
-      ||  strncmp (*bufp, "array", 5) == 0
       ||  strncmp (*bufp, "picture", 7) == 0
 #ifdef IGNOREBIB
       ||  strncmp (*bufp, "thebibliography", 15) == 0
@@ -703,7 +682,7 @@ int TeX_math_check (cont_char, bufp)
     return 0;
     }
 
-TeX_skip_parens (bufp)
+static void TeX_skip_parens (bufp)
     char **	bufp;
     {
 
@@ -711,14 +690,14 @@ TeX_skip_parens (bufp)
 	(*bufp)++;
     }
 
-TeX_open_paren (bufp)
+static void TeX_open_paren (bufp)
     char **	bufp;
     {
     while (**bufp  &&  **bufp != TEXLEFTCURLY)
 	(*bufp)++;
     }
 
-TeX_skip_check (bufp)
+static void TeX_skip_check (bufp)
     char **	bufp;
     {
     int		charlen;
@@ -731,30 +710,30 @@ TeX_skip_check (bufp)
      * setlength, addtolength, settowidth
      */
 
-    if (strncmp(*bufp, "end", 3) == 0
-      ||  strncmp(*bufp, "vspace", 6) == 0
-      ||  strncmp(*bufp, "hspace", 6) == 0
-      ||  strncmp(*bufp, "cite", 4) == 0
-      ||  strncmp(*bufp, "ref", 3) == 0
-      ||  strncmp(*bufp, "parbox", 6) == 0
-      ||  strncmp(*bufp, "label", 5) == 0
-      ||  strncmp(*bufp, "input", 5) == 0
-      ||  strncmp(*bufp, "nocite", 6) == 0
-      ||  strncmp(*bufp, "include", 7) == 0
-      ||  strncmp(*bufp, "includeonly", 11) == 0
-      ||  strncmp(*bufp, "documentstyle", 13) == 0
+    if (TeX_strncmp(*bufp, "end", 3) == 0
+      ||  TeX_strncmp(*bufp, "vspace", 6) == 0
+      ||  TeX_strncmp(*bufp, "hspace", 6) == 0
+      ||  TeX_strncmp(*bufp, "cite", 4) == 0
+      ||  TeX_strncmp(*bufp, "ref", 3) == 0
+      ||  TeX_strncmp(*bufp, "parbox", 6) == 0
+      ||  TeX_strncmp(*bufp, "label", 5) == 0
+      ||  TeX_strncmp(*bufp, "input", 5) == 0
+      ||  TeX_strncmp(*bufp, "nocite", 6) == 0
+      ||  TeX_strncmp(*bufp, "include", 7) == 0
+      ||  TeX_strncmp(*bufp, "includeonly", 11) == 0
+      ||  TeX_strncmp(*bufp, "documentstyle", 13) == 0
 #ifndef IGNOREBIB
-      ||  strncmp(*bufp, "bibliography", 12) == 0
-      ||  strncmp(*bufp, "bibitem", 7) == 0
+      ||  TeX_strncmp(*bufp, "bibliography", 12) == 0
+      ||  TeX_strncmp(*bufp, "bibitem", 7) == 0
 #endif
-      ||  strncmp(*bufp, "hyphenation", 11) == 0
-      ||  strncmp(*bufp, "pageref", 7) == 0)
+      ||  TeX_strncmp(*bufp, "hyphenation", 11) == 0
+      ||  TeX_strncmp(*bufp, "pageref", 7) == 0)
 	{
 	TeX_skip_parens (bufp);
 	if (**bufp == 0)
 	    LaTeX_Mode = 'r';
 	}
-    else if (strncmp(*bufp, "rule", 4) == 0)	/* skip two args. */
+    else if (TeX_strncmp(*bufp, "rule", 4) == 0)	/* skip two args. */
 	{
 	TeX_skip_parens (bufp);
 	if (**bufp == 0)	/* Only skips one {} if not on same line. */
@@ -789,3 +768,31 @@ TeX_skip_check (bufp)
 	    }
 	}
     }
+
+/*
+ * TeX_strncmp is like strncmp, except that it returns inequality if
+ * the following character of a is not alphabetic.  We do not use
+ * iswordch here because TeX itself won't normally accept
+ * nonalphabetics (except maybe on ISO Latin-1 installations?  I'll
+ * have to look into that).  As a special hack, because LaTeX uses the
+ * @ sign so much, we'll also accept that character.
+ *
+ * Properly speaking, the @ sign should be settable in the hash file
+ * header, but I doubt that it varies, and I don't want to change the
+ * syntax of affix files right now.
+ */
+static int TeX_strncmp (a, b, n)
+    char *	a;		/* Strings to compare */
+    char *	b;		/* ... */
+    int		n;		/* Number of characters to compare */
+    {
+    int		cmpresult;	/* Result of calling strncmp */
+
+    cmpresult = strncmp (a, b, n);
+    if (cmpresult == 0)
+	{
+	if (isascii (a[n])  &&  isalpha (a[n]))
+	    return 1;		/* Force inequality if alpha follows */
+	}
+    return cmpresult;
+    }
