@@ -1,183 +1,88 @@
-/* -*- Mode: Text -*- */
-
 /*
- * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
+ * $Id: ispell.h,v 1.60 1994/05/17 06:44:15 geoff Exp $
  */
 
 /*
- * $Id: ispell.h,v 1.41 1991/12/09 00:41:05 geoff Exp $
+ * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
  *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+
+/*
  * $Log: ispell.h,v $
- * Revision 1.41  1991/12/09  00:41:05  geoff
- * Explicitly initialize tempfile to the null string, since we depend on
- * this initialization.  (Not strictly necessary, but I prefer to make it
- * obvious that this initial value is required.)
+ * Revision 1.60  1994/05/17  06:44:15  geoff
+ * Add support for controlled compound formation and the COMPOUNDONLY
+ * option to affix flags.
+ *
+ * Revision 1.59  1994/03/15  06:25:16  geoff
+ * Change deftflag's initialization so we can tell if -t/-n appeared.
+ *
+ * Revision 1.58  1994/02/07  05:53:28  geoff
+ * Add typecasts to the the 7-bit versions of ichar* routines
+ *
+ * Revision 1.57  1994/01/25  07:11:48  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
- * Revision 1.40  91/09/12  00:01:47  geoff
- * Add definitions and header-file fields for the new TeX special
- * characters.
- * 
- * Revision 1.39  91/09/11  23:22:46  geoff
- * Add TeX_comment.
- * 
- * Revision 1.38  91/08/10  16:38:30  geoff
- * Add vflag.  Remove the typecasts in the ctype-style stuff, since they
- * were unneeded and wrong.
- * 
- * Revision 1.37  91/08/10  14:09:36  geoff
- * Fix all of the ctype-style macros to properly typecast the character
- * before testing it.  Remove the lengthofchar macro.  Change the
- * *isstringch macros to allow stringchars to be only one character long.
- * 
- * Revision 1.36  91/07/15  19:27:03  geoff
- * Change all of the stringchar macros (*isstringch and lengthofchar) to
- * accept a "canonical" parameter and pass it to stringcharlen.
- * 
- * Revision 1.35  91/07/11  19:52:14  geoff
- * Include stdio.h for the definition of BUFSIZ.
- * 
- * Revision 1.34  91/07/05  19:51:45  geoff
- * Add a proper declaration of done(), and remove the unused vb variable.
- * 
- * Revision 1.33  91/07/03  18:20:50  geoff
- * Add the chartoichar macro for conversion between chars and ichar_t's.
- * Add the "ti" and "te" terminal capabilities.
- * 
- * Revision 1.32  91/05/27  21:47:58  geoff
- * Add data-structure support for the string-character type tables.
- * 
- * Revision 1.31  90/12/31  00:59:47  geoff
- * Reformat to follow a consistent convention throughout ispell
- * 
- * Revision 1.30  90/10/14  01:51:39  geoff
- * Add the "changes" and "readonly" variables.
- * 
- * Revision 1.29  90/04/26  22:44:06  geoff
- * Add fields and variables needed to support alternate string characters.
- * 
- * Revision 1.28  89/12/27  22:29:08  geoff
- * Add minword.
- * 
- * Revision 1.27  89/07/11  00:24:11  geoff
- * Add Amiga and REGEX_LOOKUP support from luis@rice.edu.
- * 
- * Revision 1.26  89/07/07  23:26:29  geoff
- * Add a complete set of external declarations, including using the
- * new INIT macro to handle initialized declarations.
- * 
- * Revision 1.25  89/06/09  15:54:45  geoff
- * Add some lint ifdefs, so that the various macros can be checked by
- * lint.  Move the isXXXX macros to ispell.h from config.h, since they
- * shouldn't be changed during configuration.  Add support for the
- * internal "character" type, ichar_t.
- * 
- * Revision 1.24  89/04/28  01:11:16  geoff
- * Change Header to Id;  nobody cares about my pathnames.
- * 
- * Revision 1.23  89/04/03  01:56:34  geoff
- * Add support for string characters.  Fix the magic number to be constant,
- * and put the compilations in the hash header separately.
- * 
- * Revision 1.22  89/02/22  23:16:18  geoff
- * Add "contextsize" to the list of globals.
- * 
- * Revision 1.21  89/02/20  22:10:36  geoff
- * Define ALLFLAGS even if CAPITALIZATION is turned off.
- * 
- * Revision 1.20  89/02/06  02:21:38  geoff
- * Fix the offsets of TEX's left and right curlies and parens to match
- * what the documentation specifies for the "texchars" statement.
- * 
- * Revision 1.19  88/12/26  02:29:52  geoff
- * Add a copyright notice.
- * 
- * Revision 1.18  88/11/25  19:52:38  geoff
- * Fix the SET/CLR/TSTMASKBIT macros to work correctly with masks of more
- * than 32 bits.
- * 
- * Revision 1.17  88/06/25  17:50:05  geoff
- * Add hash header fields for the defaults set in the affix file:  nroff
- * and troff control characters, compound word (missing space) flag, and
- * automatic affix-generation (try hard) flag.  Update the magic number
- * to reflect the change in hash formats.
- * 
- * Revision 1.16  88/04/30  22:14:12  geoff
- * Get rid of a bunch of unused termcap variables detected by lint.
- * 
- * Revision 1.15  88/02/20  23:12:48  geoff
- * Expand the handling of MASKBITS to provide more size options.  Redo the
- * capitalization handling structures.  Change the magic number and its
- * creation so that binary-incompatible configuration options will cause
- * differing magic numbers.  Add "struct success" to describe spelling
- * hits.  Add some new externals and delete some unused one.
- * 
- * Revision 1.14  87/09/30  23:30:51  geoff
- * Make the sort order a short so it can go past the range of a char.  Add
- * a bunch of globals that used to be scattered through .c files, plus
- * one or two new ones to support the "nopossibilities" stuff.
- * 
- * Revision 1.13  87/09/09  00:18:45  geoff
- * Add sortval and sortorder to the hash header.
- * 
- * Revision 1.12  87/09/03  23:17:30  geoff
- * Get rid of a spurious WORDLEN definition, and use a different magic
- * number for NO8BIT hash files.
- * 
- * Revision 1.11  87/07/20  23:22:44  geoff
- * Major changes to support table driving.  Highlights include macros
- * to support the new flag bits, removal of the X_flag fields from the
- * table entry and replacement with the lumped masks, addition of language
- * tables, and using a different magic number if CAPITALIZE is defined.
- * 
- * Revision 1.10  87/06/27  12:20:32  geoff
- * Add the "sg" termcap entry for braindamaged terminals.
- * 
- * Revision 1.9  87/04/19  22:52:54  geoff
- * Add fields for capitalization handling.  Update MAGIC so it will refuse to
- * work with uncapitalized hash files.  Make hashheader a global.
- * 
- * Revision 1.8  87/04/01  15:22:48  geoff
- * Integrate Joe Orost's V7/register changes into the main branch
- * 
- * Revision 1.7  87/03/22  23:56:32  geoff
- * Fix minor bugs found by Dave Mason
- * 
- * Revision 1.6  87/03/01  00:56:50  geoff
- * Get rid of struct node;  add keep flag to struct dent, and delete
- * some excess stuff caused by bad patching.
- * 
- * Revision 1.5  87/02/28  14:58:21  geoff
- * Add a full struct dent to the tree node structure.
- * 
- * Revision 1.4  87/02/26  00:31:07  geoff
- * Integrate McQueer's enhancements into the main branch
- * 
- * Revision 1.3  87/01/19  00:06:41  geoff
- * Move LIBDIR to where I keep it.
- * 
- * Revision 1.2  87/01/17  13:11:53  geoff
- * Add RCS ID keywords
- * 
  */
 
 #include <stdio.h>
 
+#ifdef __STDC__
+#define P(x)	x
+#define VOID	void
+#else /* __STDC__ */
+#define P(x)	()
+#define VOID	char
+#define const
+#endif /* __STDC__ */
+
 #ifdef NO8BIT
 #define SET_SIZE	128
 #else
 #define SET_SIZE	256
 #endif
 
-#define MASKTYPE	long
-#define MASKTYPE_WIDTH	(8 * sizeof (MASKTYPE))
 #define MASKSIZE	(MASKBITS / MASKTYPE_WIDTH)
 
+#ifdef lint
+extern void	SETMASKBIT P ((MASKTYPE * mask, int bit));
+extern void	CLRMASKBIT P ((MASKTYPE * mask, int bit));
+extern int	TSTMASKBIT P ((MASKTYPE * mask, int bit));
+#else /* lint */
 /* The following is really testing for MASKSIZE <= 1, but cpp can't do that */
-#ifndef lint
-#if MASKBITS <= 32
+#if MASKBITS <= MASKTYPE_WIDTH
 #define SETMASKBIT(mask, bit) ((mask)[0] |= 1 << (bit))
 #define CLRMASKBIT(mask, bit) ((mask)[0] &= ~(1 << (bit)))
 #define TSTMASKBIT(mask, bit) ((mask)[0] & (1 << (bit)))
@@ -198,6 +103,11 @@
 #define FULLMASKSET
 #endif
 
+#ifdef lint
+extern int	BITTOCHAR P ((int bit));
+extern int	CHARTOBIT P ((int ch));
+#endif /* lint */
+
 #if MASKBITS <= 32
 # ifndef lint
 #define BITTOCHAR(bit)	((bit) + 'A')
@@ -235,10 +145,10 @@
 
 #ifdef ICHAR_IS_CHAR
 typedef unsigned char	ichar_t;	/* Internal character */
-#define icharlen	strlen
-#define icharcpy	strcpy
-#define icharcmp	strcmp
-#define icharncmp	strncmp
+#define icharlen(s)	strlen ((char *) s)
+#define icharcpy(a, b)	strcpy ((char *) a, (char *) b)
+#define icharcmp(a, b)	strcmp ((char *) a, (char *) b)
+#define icharncmp(a, b, n) strncmp ((char *) a, (char *) b, n)
 #define chartoichar(x)	((ichar_t) (x))
 #else
 typedef unsigned short	ichar_t;	/* Internal character */
@@ -259,7 +169,7 @@ struct dent
 ** Flags in the directory entry.  If FULLMASKSET is undefined, these are
 ** stored in the highest bits of the last longword of the mask field.  If
 ** FULLMASKSET is defined, they are stored in the extra "flags" field.
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 **
 ** If a word has only one capitalization form, and that form is not
 ** FOLLOWCASE, it will have exactly one entry in the dictionary.  The
@@ -328,7 +238,9 @@ struct dent
 #endif
 #define USED		(1 << (FLAGBASE + 0))
 #define KEEP		(1 << (FLAGBASE + 1))
-#ifdef CAPITALIZATION
+#ifdef NO_CAPITALIZATION_SUPPORT
+#define ALLFLAGS	(USED | KEEP)
+#else /* NO_CAPITALIZATION_SUPPORT */
 #define ANYCASE		(0 << (FLAGBASE + 2))
 #define ALLCAPS		(1 << (FLAGBASE + 2))
 #define CAPITALIZED	(2 << (FLAGBASE + 2))
@@ -337,9 +249,7 @@ struct dent
 #define MOREVARIANTS	(1 << (FLAGBASE + 4))
 #define ALLFLAGS	(USED | KEEP | CAPTYPEMASK | MOREVARIANTS)
 #define captype(x)	((x) & CAPTYPEMASK)
-#else /* CAPITALIZATION */
-#define ALLFLAGS	(USED | KEEP)
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
  * Language tables used to encode prefix and suffix information.
@@ -360,6 +270,7 @@ struct flagent
  * Bits in flagflags
  */
 #define FF_CROSSPRODUCT	(1 << 0)		/* Affix does cross-products */
+#define FF_COMPOUNDONLY	(1 << 1)		/* Afx works in compounds */
 
 union ptr_union					/* Aid for building flg ptrs */
     {
@@ -379,6 +290,7 @@ struct flagptr
 struct strchartype
     {
     char *		name;			/* Name of the type */
+    char *		deformatter;		/* Deformatter to use */
     char *		suffixes;		/* File suffixes, null seps */
     };
 
@@ -387,10 +299,12 @@ struct strchartype
  */
 struct hashheader
     {
-    short magic;    	    	    	    	/* Magic number for ID */
-    short compileoptions;			/* How we were compiled */
+    unsigned short magic;    	    	    	/* Magic number for ID */
+    unsigned short compileoptions;		/* How we were compiled */
     short maxstringchars;			/* Max # strchrs we support */
     short maxstringcharlen;			/* Max strchr len supported */
+    short compoundmin;				/* Min lth of compound parts */
+    short compoundbit;				/* Flag 4 compounding roots */
     int stringsize;				/* Size of string table */
     int lstringsize;				/* Size of lang. str tbl */
     int tblsize;				/* No. entries in hash tbl */
@@ -402,7 +316,7 @@ struct hashheader
     int strtypestart;				/* Start of strtype table */
     char nrchars[5];				/* Nroff special characters */
     char texchars[13];				/* TeX special characters */
-    char defspaceflag;				/* Default missingspace flag */
+    char compoundflag;				/* Compund-word handling */
     char defhardflag;				/* Default tryveryhard flag */
     char flagmarker;				/* "Start-of-flags" char */
     unsigned short sortorder[SET_SIZE + MAXSTRINGCHARS]; /* Sort ordering */
@@ -416,10 +330,11 @@ struct hashheader
     char stringchars[MAXSTRINGCHARS][MAXSTRINGCHARLEN + 1]; /* String chars */
     char stringdups[MAXSTRINGCHARS];		/* No. of "base" char */
     char dupnos[MAXSTRINGCHARS];		/* Dup char ID # */
+    unsigned short magic2;			/* Second magic for dbl chk */
     };
 
 /* hash table magic number */
-#define MAGIC			0x9601
+#define MAGIC			0x9602
 
 /* compile options, put in the hash header for consistency checking */
 #ifdef NO8BIT
@@ -427,10 +342,10 @@ struct hashheader
 #else
 # define MAGIC8BIT		0x00
 #endif
-#ifdef CAPITALIZATION
-# define MAGICCAPITALIZATION	0x02
-#else
+#ifdef NO_CAPITALIZATION_SUPPORT
 # define MAGICCAPITALIZATION	0x00
+#else
+# define MAGICCAPITALIZATION	0x02
 #endif
 #if MASKBITS <= 32
 # define MAGICMASKSET		0x00
@@ -485,14 +400,27 @@ struct success
 #define TEXDOT		hashheader.texchars[11]
 #define TEXPERCENT	hashheader.texchars[12]
 
+/*
+** Values for compoundflag
+*/
+#define COMPOUND_NEVER		0	/* Compound words are never good */
+#define COMPOUND_ANYTIME	1	/* Accept run-together words */
+#define COMPOUND_CONTROLLED	2	/* Compounds controlled by afx flags */
+
 /*
 ** The isXXXX macros normally only check ASCII range, and don't support
 ** the character sets of other languages.  These private versions handle
 ** whatever character sets have been defined in the affix files.
 */
 #ifdef lint
-extern ichar_t mytolower ();
-extern ichar_t mytoupper ();
+extern int	myupper P ((unsigned int ch));
+extern int	mylower P ((unsigned int ch));
+extern int	myspace P ((unsigned int ch));
+extern int	iswordch P ((unsigned int ch));
+extern int	isboundarych P ((unsigned int ch));
+extern int	isstringstart P ((unsigned int ch));
+extern ichar_t	mytolower P ((unsigned int ch));
+extern ichar_t	mytoupper P ((unsigned int ch));
 #else /* lint */
 #define myupper(X)	(hashheader.upperchars[X])
 #define mylower(X)	(hashheader.lowerchars[X])
@@ -531,15 +459,12 @@ extern ichar_t mytoupper ();
 					> 0 \
 				      ? 1 : (len = 1, 0)))
 
-#ifdef AMIGA
-#ifndef BUILDHASH
-#define printf		printcon
-#undef putchar
-#undef getchar
-#define putchar 	putccon
-#define getchar 	getccon
-#endif /* BUILDHASH */
-#endif /* AMIGA */
+/*
+ * Sizes of buffers returned by ichartosstr/strtosichar.
+ */
+#define ICHARTOSSTR_SIZE (INPUTWORDLEN + 4 * MAXAFFIXLEN + 4)
+#define STRTOSICHAR_SIZE ((INPUTWORDLEN + 4 * MAXAFFIXLEN + 4) \
+			  * sizeof (ichar_t))
 
 /*
  * termcap variables
@@ -550,23 +475,6 @@ extern ichar_t mytoupper ();
 # define EXTERN extern
 #endif
 
-extern SIGNAL_TYPE	done ();
-extern char *		tgetstr ();
-extern char *		ichartosstr ();
-extern ichar_t *	strtosichar ();
-extern char *		printichar ();
-extern char *		getline ();
-extern void		chupcase ();
-extern void		upcase ();
-extern void 		lowcase ();
-#ifdef REGEX_LOOKUP
-extern char *		do_regex_lookup ();
-#endif /* REGEX_LOOKUP */
-
-#ifdef AMIGA 
-extern long		_Heapsize;
-#endif /* AMIGA */
-
 EXTERN char *	BC;	/* backspace if not ^H */
 EXTERN char *	cd;	/* clear to end of display */
 EXTERN char *	cl;	/* clear display */
@@ -587,8 +495,8 @@ EXTERN char *	currentchar;	/* Location in contextbufs */
 EXTERN char	ctoken[INPUTWORDLEN + MAXAFFIXLEN]; /* Current token as char */
 EXTERN ichar_t	itoken[INPUTWORDLEN + MAXAFFIXLEN]; /* Ctoken as ichar_t str */
 
-EXTERN char	termcap[1024];	/* termcap entry */
-EXTERN char	termstr[1024];	/* for string values */
+EXTERN char	termcap[2048];	/* termcap entry */
+EXTERN char	termstr[2048];	/* for string values */
 EXTERN char *	termptr;	/* pointer into termcap, used by tgetstr */
 
 EXTERN int	numhits;	/* number of hits in dictionary lookups */
@@ -678,21 +586,20 @@ INIT (int sflag, 0);			/* NZ to stop self after EOF */
 #endif
 INIT (int vflag, 0);			/* NZ to display characters as M-xxx */
 INIT (int xflag, DEFNOBACKUPFLAG);	/* NZ to suppress backups */
-INIT (int deftflag, DEFTEXFLAG);	/* NZ for TeX mode by default */
+INIT (int deftflag, -1);		/* NZ for TeX mode by default */
 INIT (int tflag, DEFTEXFLAG);		/* NZ for TeX mode in current file */
 INIT (int prefstringchar, -1);		/* Preferred string character type */
 
 INIT (int terse, 0);			/* NZ for "terse" mode */
-INIT (char *LibDict, NULL);		/* Pointer to name of $(LIBDIR)/dict */
 
 INIT (char tempfile[MAXPATHLEN], "");	/* Name of file we're spelling into */
 
 INIT (int minword, MINWORD);		/* Longest always-legal word */
 INIT (int sortit, 1);			/* Sort suggestions alphabetically */
-INIT (int missingspaceflag, -1);	/* Don't report missing spaces */
+INIT (int compoundflag, -1);		/* How to treat compounds: see above */
 INIT (int tryhardflag, -1);		/* Always call tryveryhard */
 
-INIT (char *currentfile, NULL);		/* Name of current input file */
+INIT (char * currentfile, NULL);	/* Name of current input file */
 
 /* Odd numbers for math mode in LaTeX; even for LR or paragraph mode */
 INIT (int math_mode, 0);
