@@ -1,149 +1,102 @@
-/* -*- Mode:Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-	"$Id: makedent.c,v 1.28 91/08/10 14:09:40 geoff Exp $";
+	"$Id: makedent.c,v 1.42 1994/02/07 04:23:43 geoff Exp $";
 #endif
 
 /*
- * Copyright 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
+ * Copyright 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 /*
- * $Log:	makedent.c,v $
- * Revision 1.28  91/08/10  14:09:40  geoff
- * Fix stringcharlen to return zero for regular characters, so that
- * single-character stringchars can be supported.
- * 
- * Revision 1.27  91/07/15  19:27:09  geoff
- * Add comments mentioning that the input to makedent and chupcase must
- * be canonical.  Add a "canonical" parameter to strtoichar, strtosichar,
- * stringcharlen, and all calls to those routines and to l1_isstringch.
- * 
- * Revision 1.26  91/07/11  19:52:19  geoff
- * Remove the include of stdio.h, since ispell.h now does this.
- * 
- * Revision 1.25  91/07/05  20:48:59  geoff
- * Fix a typo in the previous code to obey onlykeep.
- * 
- * Revision 1.24  91/07/05  20:31:57  geoff
- * Fix some more lint complaints.  Fix toutent so it obeys "onlykeep"
- * regardless of the setting of the CAPITALIZATION option.
- * 
- * Revision 1.23  91/07/05  20:15:50  geoff
- * Don't conditionally compile ibuf;  it's always needed
- * 
- * Revision 1.22  91/07/05  19:51:49  geoff
- * Fix some lint complaints, including removing the unused "hasaffixes"
- * function.
- * 
- * Revision 1.21  91/07/04  00:04:17  geoff
- * Pick up findfiletype() from ispell.c.  Change stringcharlen to insist
- * on a match with the current string character type;  this allows
- * ambiguities between different notations to be resolved.
- * 
- * Revision 1.20  91/07/03  18:20:59  geoff
- * Replace all conversions of chars to ichar_t's with a macro call which
- * (a) does the conversion correctly and (b) makes future changes easy in
- * case (a) is false.
- * 
- * Revision 1.19  91/06/23  22:08:57  geoff
- * When casting to ichar_t, cast to unsigned first to avoid sign-extension
- * problems.
- * 
- * Revision 1.18  91/06/12  19:15:17  geoff
- * Range-check the bit number before setting mask bits.
- * 
- * Revision 1.17  91/06/11  02:26:02  geoff
- * Before allocating space for a word when making a dictionary entry,
- * canonicalize it by converting to an ichar_t and back.  Fix chupcase to
- * return the result in canonical form.  Add a warning to the header
- * comment for ichartostr, letting the user know that the length may change.
- * 
- * Revision 1.16  91/05/27  21:48:05  geoff
- * Change the string-character support to get the preferred string
- * character from a global flag based on the file type, rather than from
- * the last string processed (which may be internally-generated and thus
- * wrong).  Move a misplaced break statement.
- * 
- * Revision 1.15  90/12/31  00:59:41  geoff
- * Reformat to follow a consistent convention throughout ispell
- * 
- * Revision 1.14  90/04/26  22:44:14  geoff
- * Add support for alternate string characters.  This primarily involves changes
- * to the ichartostr and strtoichar routines.
- * 
- * Revision 1.13  90/04/17  15:34:19  geoff
- * Fix two incorrect declarations caught by gcc
- * 
- * Revision 1.12  89/12/27  03:18:22  geoff
- * Move all messages to msgs.h so they can be reconfigured
- * 
- * Revision 1.11  89/06/09  15:55:31  geoff
- * Add support for the internal "character" type, ichar_t.  This includes
- * providing the new routines chupcase, strtoichar, ichartostr, strtosichar,
- * ichartosstr, printichar, icharcpy, icharcmp, icharncmp, and icharlen.
- * Also add a little "lint library" for the various macros so that lint
- * can check macro calls for correctness too.
- * 
- * Revision 1.10  89/04/27  23:32:48  geoff
- * Add setfindlast et al, to speed up repetitive findlastchar calls.
- * 
- * Revision 1.9  89/04/03  01:57:25  geoff
- * Fix a bunch of lint complaints.  Add support for the selectable flag
- * marker character.  Add support for string characters.
- * 
- * Revision 1.8  89/02/20  22:11:40  geoff
- * Many minor upgrades to make it compile and run correctly when
- * capitalization support is turned off.  Most of these involve adding
- * more ifdef's.
- * 
- * Revision 1.7  88/12/26  02:30:46  geoff
- * Update the copyright notice.
- * 
- * Revision 1.6  88/11/16  02:19:46  geoff
- * Fix the ifdefs in combineaffixes to properly handle all sizes of MASKBITS.
- * 
- * Revision 1.5  88/04/30  22:14:46  geoff
- * Fix some lint complaints.
- * 
- * Revision 1.4  88/03/27  01:02:30  geoff
- * Add a copyright message.  Fix a curly-brace error.  Compile "combineaffixes"
- * regardless of the setting of CAPITALIZATION.
- * 
- * Revision 1.3  88/03/12  02:44:31  geoff
- * Put newlines before makedent's error messages, so they show up more
- * nicely on ispell's screen.
- * 
- * Revision 1.2  88/02/28  23:17:18  geoff
- * Combine affixes when the case and keep flag both match exactly.
- * 
- * Revision 1.1  88/02/24  00:18:23  geoff
- * Initial revision
- * 
+ * $Log: makedent.c,v $
+ * Revision 1.42  1994/02/07  04:23:43  geoff
+ * Correctly identify the deformatter when changing file types
+ *
+ * Revision 1.41  1994/01/25  07:11:55  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
+ *
  */
 
 #include "config.h"
 #include "ispell.h"
+#include "proto.h"
 #include "msgs.h"
 
-extern char *	index ();
-extern char *	strcpy ();
-
-extern char *	mymalloc ();
-extern void	myfree ();
-#ifdef CAPITALIZATION
-long		whatcap ();
+int		makedent P ((char * lbuf, int lbuflen, struct dent * ent));
+#ifndef NO_CAPITALIZATION_SUPPORT
+long		whatcap P ((ichar_t * word));
 #endif
-void		upcase ();
-void		lowcase ();
+int		addvheader P ((struct dent * ent));
+int		combinecaps P ((struct dent * hdr, struct dent * newent));
+#ifndef NO_CAPITALIZATION_SUPPORT
+static void	forcevheader P ((struct dent * hdrp, struct dent * oldp,
+		  struct dent * newp));
+#endif /* NO_CAPITALIZATION_SUPPORT */
+static int	combine_two_entries P ((struct dent * hdrp,
+		  struct dent * oldp, struct dent * newp));
+static int	acoversb P ((struct dent * enta, struct dent * entb));
+void		upcase P ((ichar_t * string));
+void		lowcase P ((ichar_t * string));
+void		chupcase P ((char * s));
+static int	issubset P ((struct dent * ent1, struct dent * ent2));
+static void	combineaffixes P ((struct dent * ent1, struct dent * ent2));
+void		toutent P ((FILE * outfile, struct dent * hent,
+		  int onlykeep));
+static void	toutword P ((FILE * outfile, char * word,
+		  struct dent * cent));
+static void	flagout P ((FILE * outfile, int flag));
+int		stringcharlen P ((char * bufp, int canonical));
+int		strtoichar P ((ichar_t * out, char * in, int outlen,
+		  int canonical));
+int		ichartostr P ((char * out, ichar_t * in, int outlen,
+		  int canonical));
+ichar_t *	strtosichar P ((char * in, int canonical));
+char *		ichartosstr P ((ichar_t * in, int canonical));
+char *		printichar P ((int in));
+#ifndef ICHAR_IS_CHAR
+ichar_t *	icharcpy P ((ichar_t * out, ichar_t * in));
+int		icharlen P ((ichar_t * str));
+int		icharcmp P ((ichar_t * s1, ichar_t * s2));
+int		icharncmp P ((ichar_t * s1, ichar_t * s2, int n));
+#endif /* ICHAR_IS_CHAR */
+int		findfiletype P ((char * name, int searchnames,
+		  int * deformatter));
 
-static void	toutword ();
-static void	flagout ();
-
-static int  	hasmarker;
+static int  	has_marker;
 
 /*
  * Fill in a directory entry, including setting the capitalization flags, and
@@ -151,8 +104,9 @@ static int  	hasmarker;
  * if there was trouble.  The input word must be in canonical form.
  */
 
-makedent (lbuf, d)
+int makedent (lbuf, lbuflen, d)
     char *		lbuf;
+    int			lbuflen;
     struct dent *	d;
     {
     ichar_t		ibuf[INPUTWORDLEN + MAXAFFIXLEN];
@@ -180,10 +134,14 @@ makedent (lbuf, d)
     ** Convert the word to an ichar_t and back;  this makes sure that
     ** it is in canonical form and thus that the length is correct.
     */
-    strtoichar (ibuf, lbuf, 1);
-    ichartostr (lbuf, ibuf, 1);
+    if (strtoichar (ibuf, lbuf, INPUTWORDLEN * sizeof (ichar_t), 1)
+      ||  ichartostr (lbuf, ibuf, lbuflen, 1))
+	{
+	(void) fprintf (stderr, WORD_TOO_LONG (lbuf));
+	return (-1);
+	}
     len = strlen (lbuf);
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
     /*
     ** Figure out the capitalization rules from the capitalization of
     ** the sample entry.
@@ -193,11 +151,11 @@ makedent (lbuf, d)
 
     if (len > INPUTWORDLEN - 1)
 	{
-	(void) fprintf (stderr, MAKEDENT_C_TOO_BIG, lbuf);
+	(void) fprintf (stderr, WORD_TOO_LONG (lbuf));
 	return (-1);
 	}
 
-    d->word = mymalloc (len + 1);
+    d->word = mymalloc ((unsigned) len + 1);
     if (d->word == NULL)
 	{
 	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, lbuf);
@@ -205,12 +163,12 @@ makedent (lbuf, d)
 	}
 
     (void) strcpy (d->word, lbuf);
-#ifdef CAPITALIZATION
+#ifdef NO_CAPITALIZATION_SUPPORT
+    chupcase (d->word);
+#else /* NO_CAPITALIZATION_SUPPORT */
     if (captype (d->flagfield) != FOLLOWCASE)
 	chupcase (d->word);
-#else /* CAPITALIZATION */
-    chupcase (d->word);
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
     if (p == NULL)
 	return (0);
 
@@ -231,11 +189,12 @@ makedent (lbuf, d)
     return (0);
     }
 
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 /*
 ** Classify the capitalization of a sample entry.  Returns one of the
 ** four capitalization codes ANYCASE, ALLCAPS, CAPITALIZED, or FOLLOWCASE.
 */
+
 long whatcap (word)
     register ichar_t *	word;
     {
@@ -281,14 +240,13 @@ long whatcap (word)
 	}
     }
 
-
 /*
 ** Add a variant-capitalization header to a word.  This routine may be
 ** called even for a followcase word that doesn't yet have a header.
 **
 ** Returns 0 if all was ok, -1 if allocation error.
 */
-addvheader (dp)
+int addvheader (dp)
     register struct dent *	dp;	/* Entry to update */
     {
     register struct dent *	tdent; /* Copy of entry */
@@ -309,7 +267,7 @@ addvheader (dp)
     else
 	{
 	/* Followcase words need a copy of the capitalization */
-	tdent->word = mymalloc (strlen (tdent->word) + 1);
+	tdent->word = mymalloc ((unsigned int) strlen (tdent->word) + 1);
 	if (tdent->word == NULL)
 	    {
 	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, dp->word);
@@ -324,7 +282,7 @@ addvheader (dp)
     dp->flagfield |= (ALLCAPS | MOREVARIANTS);
     return 0;
     }
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
 ** Combine and resolve the entries describing two capitalizations of the same
@@ -352,7 +310,7 @@ addvheader (dp)
 **	(1) Add newp's affixes and KEEP flag to oldp, and discard newp.
 **	(2) Add oldp's affixes and KEEP flag to newp, replace oldp with
 **	    newp, and discard newp.
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 **	(3) Insert newp as a new entry in the variants list.  If there is
 **	    currently no variant header, this requires adding one.  Adding a
 **	    header splits into two sub-cases:
@@ -383,7 +341,7 @@ addvheader (dp)
 **	    (4c) If the words are FOLLOWCASE, the capitalizations match
 **		exactly.
 **
-#ifdef CAPITALIZATON
+#ifndef NO_CAPITALIZATION_SUPPORT
 **	(5) For entries with mismatched capitalization types, A covers B
 **	    if (4a) and (4b) are true, and:
 **
@@ -401,11 +359,11 @@ int combinecaps (hdrp, newp)
     {
     register struct dent *
 			oldp;	/* Current "oldp" entry */
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
     register struct dent *
 			tdent; /* Entry we'll add to the dictionary */
-#endif /* CAPITALIZATION */
-    register int	retval;	/* Return value from combine_two_entries */
+#endif /* NO_CAPITALIZATION_SUPPORT */
+    register int	retval = 0; /* Return value from combine_two_entries */
 
     /*
     ** First, see if we can combine the two entries (cases 1 and 2).  If
@@ -413,7 +371,9 @@ int combinecaps (hdrp, newp)
     ** it will return zero.
     */
     oldp = hdrp;
-#ifdef CAPITALIZATION
+#ifdef NO_CAPITALIZATION_SUPPORT
+    retval = combine_two_entries (hdrp, oldp, newp);
+#else /* NO_CAPITALIZATION_SUPPORT */
     if ((oldp->flagfield & (CAPTYPEMASK | MOREVARIANTS))
       == (ALLCAPS | MOREVARIANTS))
 	{
@@ -456,18 +416,16 @@ int combinecaps (hdrp, newp)
 	    myfree (newp->word);		/* newp->word isn't needed */
 	    }
 	}
-#else /* CAPITALIZATION */
-    retval = combine_two_entries (hdrp, oldp, newp);
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
     return retval;
     }
 
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 /*
 ** The following routine implements steps 3a and 3b in the commentary
 ** for "combinecaps".
 */
-int forcevheader (hdrp, oldp, newp)
+static void forcevheader (hdrp, oldp, newp)
     register struct dent *	hdrp;
     struct dent *		oldp;
     struct dent *		newp;
@@ -480,14 +438,14 @@ int forcevheader (hdrp, oldp, newp)
       != (ALLCAPS | MOREVARIANTS))
 	(void) addvheader (hdrp);
     }
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
 ** This routine implements steps 4 and 5 of the commentary for "combinecaps".
 **
 ** Returns 1 if newp can be discarded, 0 if nothing done.
 */
-int combine_two_entries (hdrp, oldp, newp)
+static int combine_two_entries (hdrp, oldp, newp)
     struct dent *	hdrp;	/* (Possible) header of variant chain */
     register struct dent *
 			oldp;	/* Pre-existing dictionary entry */
@@ -511,11 +469,11 @@ int combine_two_entries (hdrp, oldp, newp)
 	** the keep flag.
 	*/
 	combineaffixes (newp, oldp);
-#ifdef CAPITALIZATION
-	newp->flagfield |= (oldp->flagfield & (KEEP | MOREVARIANTS));
-#else /* CAPITALIZATION */
+#ifdef NO_CAPITALIZATION_SUPPORT
 	newp->flagfield |= (oldp->flagfield & KEEP);
-#endif /* CAPITALIZATION */
+#else /* NO_CAPITALIZATION_SUPPORT */
+	newp->flagfield |= (oldp->flagfield & (KEEP | MOREVARIANTS));
+#endif /* NO_CAPITALIZATION_SUPPORT */
 	hdrp->flagfield |= (newp->flagfield & KEEP);
 	newp->next = oldp->next;
 	/*
@@ -529,13 +487,13 @@ int combine_two_entries (hdrp, oldp, newp)
 	myfree (newp->word);	/* No longer needed */
 	newp->word = oldp->word;
 	*oldp = *newp;
-#ifdef CAPITALIZATION
+#ifndef NO_CAPITALIZATION_SUPPORT
 	/* We may need to add a header if newp is followcase */
 	if (captype (newp->flagfield) == FOLLOWCASE
 	  &&  (hdrp->flagfield & (CAPTYPEMASK | MOREVARIANTS))
 	    != (ALLCAPS | MOREVARIANTS))
 	    (void) addvheader (hdrp);
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
 	return 1;
 	}
     else
@@ -546,7 +504,7 @@ int combine_two_entries (hdrp, oldp, newp)
 ** Determine if enta covers entb, according to the rules in steps 4 and 5
 ** of the commentary for "combinecaps".
 */
-int acoversb (enta, entb)
+static int acoversb (enta, entb)
     register struct dent *	enta;	/* "A" in the rules */
     register struct dent *	entb;	/* "B" in the rules */
     {
@@ -561,13 +519,19 @@ int acoversb (enta, entb)
 	}
     else
 	{
-	/* not a subset;  KEEP flags must match exactly (both (4a) and (4b) */
+	/* not a subset;  KEEP flags must match exactly (both (4a) and (4b)) */
 	if (((enta->flagfield ^ entb->flagfield) & KEEP) != 0)
 	    return 0;
 	}
 
     /* Rules (4a) and (4b) are satisfied;  check for capitalization match */
-#ifdef CAPITALIZATION
+#ifdef NO_CAPITALIZATION_SUPPORT
+#ifdef lint
+    return subset;				/* Just so it gets used */
+#else /* lint */
+    return 1;					/* All words match */
+#endif /* lint */
+#else /* NO_CAPITALIZATION_SUPPORT */
     if (((enta->flagfield ^ entb->flagfield) & CAPTYPEMASK) == 0)
 	{
 	if (captype (enta->flagfield) != FOLLOWCASE	/* Condition (4c) */
@@ -585,13 +549,7 @@ int acoversb (enta, entb)
 	return 1;
     else
 	return 0;
-#else /* CAPITALIZATION */
-#ifdef lint
-    return subset;				/* Just so it gets used */
-#else /* lint */
-    return 1;					/* All words match */
-#endif /* lint */
-#endif /* CAPITALIZATION */
+#endif /* NO_CAPITALIZATION_SUPPORT */
     }
 
 void upcase (s)
@@ -627,14 +585,14 @@ void chupcase (s)
 
     is = strtosichar (s, 1);
     upcase (is);
-    ichartostr (s, is, 1);
+    (void) ichartostr (s, is, strlen (s) + 1, 1);
     }
 
 /*
 ** See if one affix field is a subset of another.  Returns NZ if ent1
 ** is a subset of ent2.  The KEEP flag is not taken into consideration.
 */
-int issubset (ent1, ent2)
+static int issubset (ent1, ent2)
     register struct dent *	ent1;
     register struct dent *	ent2;
     {
@@ -669,7 +627,7 @@ int issubset (ent1, ent2)
 /*
 ** Add ent2's affix flags to ent1.
 */
-combineaffixes (ent1, ent2)
+static void combineaffixes (ent1, ent2)
     register struct dent *	ent1;
     register struct dent *	ent2;
     {
@@ -693,18 +651,22 @@ combineaffixes (ent1, ent2)
 ** If onlykeep is true, only those variants with KEEP set will be
 ** written.
 */
-toutent (outfile, hent, onlykeep)
-    register FILE *	outfile;
+void toutent (toutfile, hent, onlykeep)
+    register FILE *	toutfile;
     struct dent *	hent;
     register int	onlykeep;
     {
-#ifdef CAPITALIZATION
+#ifdef NO_CAPITALIZATION_SUPPORT
+    if (!onlykeep  ||  (hent->flagfield & KEEP))
+	toutword (toutfile, hent->word, hent);
+#else
     register struct dent * cent;
     ichar_t		wbuf[INPUTWORDLEN + MAXAFFIXLEN];
 
     cent = hent;
-    strtoichar (wbuf, cent->word, 1);
-    while (1)
+    if (strtoichar (wbuf, cent->word, INPUTWORDLEN, 1))
+	(void) fprintf (stderr, WORD_TOO_LONG (cent->word));
+    for (  ;  ;  )
 	{
 	if (!onlykeep  ||  (cent->flagfield & KEEP))
 	    {
@@ -712,23 +674,23 @@ toutent (outfile, hent, onlykeep)
 		{
 		case ANYCASE:
 		    lowcase (wbuf);
-		    toutword (outfile, ichartosstr (wbuf, 1), cent);
+		    toutword (toutfile, ichartosstr (wbuf, 1), cent);
 		    break;
 		case ALLCAPS:
 		    if ((cent->flagfield & MOREVARIANTS) == 0
 		      ||  cent != hent)
 			{
 			upcase (wbuf);
-			toutword (outfile, ichartosstr (wbuf, 1), cent);
+			toutword (toutfile, ichartosstr (wbuf, 1), cent);
 			}
 		    break;
 		case CAPITALIZED:
 		    lowcase (wbuf);
 		    wbuf[0] = mytoupper (wbuf[0]);
-		    toutword (outfile, ichartosstr (wbuf, 1), cent);
+		    toutword (toutfile, ichartosstr (wbuf, 1), cent);
 		    break;
 		case FOLLOWCASE:
-		    toutword (outfile, cent->word, cent);
+		    toutword (toutfile, cent->word, cent);
 		    break;
 		}
 	    }
@@ -737,37 +699,34 @@ toutent (outfile, hent, onlykeep)
 	else
 	    break;
 	}
-#else
-    if (!onlykeep  ||  (hent->flagfield & KEEP))
-	toutword (outfile, hent->word, hent);
 #endif
     }
 		
-static void toutword (outfile, word, cent)
-    register FILE *	outfile;
+static void toutword (toutfile, word, cent)
+    register FILE *	toutfile;
     char *		word;
     register struct dent * cent;
     {
     register int	bit;
 
-    hasmarker = 0;
-    (void) fprintf (outfile, "%s", word);
+    has_marker = 0;
+    (void) fprintf (toutfile, "%s", word);
     for (bit = 0;  bit < LARGESTFLAG;  bit++)
 	{
 	if (TSTMASKBIT (cent->mask, bit))
-	  flagout (outfile, BITTOCHAR (bit));
+	  flagout (toutfile, BITTOCHAR (bit));
 	}
-    (void) fprintf (outfile, "\n");
+    (void) fprintf (toutfile, "\n");
     }
 
-static void flagout (outfile, flag)
-    register FILE *	outfile;
+static void flagout (toutfile, flag)
+    register FILE *	toutfile;
     int			flag;
     {
-    if (!hasmarker)
-	(void) putc (hashheader.flagmarker, outfile);
-    hasmarker = 1;
-    (void) putc (flag, outfile);
+    if (!has_marker)
+	(void) putc (hashheader.flagmarker, toutfile);
+    has_marker = 1;
+    (void) putc (flag, toutfile);
     }
 
 /*
@@ -868,15 +827,19 @@ int stringcharlen (bufp, canonical)
 /*
  * Convert an external string to an ichar_t string.  If necessary, the parity
  * bit is stripped off as part of the process.
+ *
+ * Returns NZ if the output string overflowed.
  */
-strtoichar (out, in, canonical)
+int strtoichar (out, in, outlen, canonical)
     register ichar_t *	out;		/* Where to put result */
     register char *	in;		/* String to convert */
+    int			outlen;		/* Size of output buffer, *BYTES* */
     int			canonical;	/* NZ if input is in canonical form */
     {
     register int	len;		/* Length of next character */
 
-    for (  ;  *in != '\0';  in += len)
+    outlen /= sizeof (ichar_t);		/* Convert to an ichar_t count */
+    for (  ;  --outlen > 0  &&  *in != '\0';  in += len)
 	{
 	if (l1_isstringch (in, len, canonical))
 	    *out++ = SET_SIZE + laststringch;
@@ -884,28 +847,33 @@ strtoichar (out, in, canonical)
 	    *out++ = *in & NOPARITY;
 	}
     *out = 0;
+    return outlen <= 0;
     }
 
 /*
- * Convert an ichar_t string to an external string.  WARNING:  the
- * resulting string may wind up being longer than the original.  In
- * fact, even the sequence strtoichar->ichartostr may produce a
- * result longer than the original, because the output form may use
- * a different string type set than the original input form.
+ * Convert an ichar_t string to an external string.
+ *
+ * WARNING: the resulting string may wind up being longer than the
+ * original.  In fact, even the sequence strtoichar->ichartostr may
+ * produce a result longer than the original, because the output form
+ * may use a different string type set than the original input form.
+ *
+ * Returns NZ if the output string overflowed.
  */
-ichartostr (out, in, canonical)
+int ichartostr (out, in, outlen, canonical)
     register char *	out;		/* Where to put result */
     register ichar_t *	in;		/* String to convert */
+    int			outlen;		/* Size of output buffer, bytes */
     int			canonical;	/* NZ for canonical form */
     {
     register int	ch;		/* Next character to store */
     register int	i;		/* Index into duplicates list */
     register char *	scharp;		/* Pointer into a string char */
 
-    while ((ch = *in++) != 0)
+    while (--outlen > 0  &&  (ch = *in++) != 0)
 	{
 	if (ch < SET_SIZE)
-	    *out++ = ch;
+	    *out++ = (char) ch;
 	else
 	    {
 	    ch -= SET_SIZE;
@@ -928,6 +896,7 @@ ichartostr (out, in, canonical)
 	    }
 	}
     *out = '\0';
+    return outlen <= 0;
     }
 
 /*
@@ -937,9 +906,10 @@ ichar_t * strtosichar (in, canonical)
     char *		in;		/* String to convert */
     int			canonical;	/* NZ if input is in canonical form */
     {
-    static ichar_t	out[INPUTWORDLEN + 4 * MAXAFFIXLEN + 4];
+    static ichar_t	out[STRTOSICHAR_SIZE / sizeof (ichar_t)];
 
-    strtoichar (out, in, canonical);
+    if (strtoichar (out, in, sizeof out, canonical))
+	(void) fprintf (stderr, WORD_TOO_LONG (in));
     return out;
     }
 
@@ -950,9 +920,10 @@ char * ichartosstr (in, canonical)
     ichar_t *		in;		/* Internal string to convert */
     int			canonical;	/* NZ for canonical conversion */
     {
-    static char		out[INPUTWORDLEN + 4 * MAXAFFIXLEN + 4];
+    static char		out[ICHARTOSSTR_SIZE];
 
-    ichartostr (out, in, canonical);
+    if (ichartostr (out, in, sizeof out, canonical))
+	(void) fprintf (stderr, WORD_TOO_LONG (out));
     return out;
     }
 
@@ -961,13 +932,13 @@ char * ichartosstr (in, canonical)
  * a static area.
  */
 char * printichar (in)
-    ichar_t		in;
+    int			in;
     {
     static char		out[MAXSTRINGCHARLEN + 1];
 
     if (in < SET_SIZE)
 	{
-	out[0] = in;
+	out[0] = (char) in;
 	out[1] = '\0';
 	}
     else
@@ -979,19 +950,22 @@ char * printichar (in)
 /*
  * Copy an ichar_t.
  */
-icharcpy (out, in)
-    register ichar_t *	in;		/* Source */
+ichar_t * icharcpy (out, in)
     register ichar_t *	out;		/* Destination */
+    register ichar_t *	in;		/* Source */
     {
+    ichar_t *		origout;	/* Copy of destination for return */
 
+    origout = out;
     while ((*out++ = *in++) != 0)
 	;
+    return origout;
     }
 
 /*
  * Return the length of an ichar_t.
  */
-icharlen (in)
+int icharlen (in)
     register ichar_t *	in;		/* String to count */
     {
     register int	len;		/* Length so far */
@@ -1004,7 +978,7 @@ icharlen (in)
 /*
  * Compare two ichar_t's.
  */
-icharcmp (s1, s2)
+int icharcmp (s1, s2)
     register ichar_t *	s1;
     register ichar_t *	s2;
     {
@@ -1020,7 +994,7 @@ icharcmp (s1, s2)
 /*
  * Strncmp for two ichar_t's.
  */
-icharncmp (s1, s2, n)
+int icharncmp (s1, s2, n)
     register ichar_t *	s1;
     register ichar_t *	s2;
     register int	n;
@@ -1039,30 +1013,46 @@ icharncmp (s1, s2, n)
 
 #endif /* ICHAR_IS_CHAR */
 
-findfiletype (name, searchnames)
+int findfiletype (name, searchnames, deformatter)
     char *		name;		/* Name to look up in suffix table */
     int			searchnames;	/* NZ to search name field of table */
+    int *		deformatter;	/* Where to set deformatter type */
     {
     char *		cp;		/* Pointer into suffix list */
     int			cplen;		/* Length of current suffix */
     register int	i;		/* Index into type table */
     int			len;		/* Length of the name */
 
+    /*
+     * Note:  for now, the deformatter is set to 1 for tex, 0 for nroff.
+     * Further, we assume that it's one or the other, so that a test
+     * for tex is sufficient.  This needs to be generalized.
+     */
     len = strlen (name);
-    for (i = 0;  i < hashheader.nstrchartype;  i++)
+    if (searchnames)
 	{
-	if (searchnames)
+	for (i = 0;  i < hashheader.nstrchartype;  i++)
 	    {
 	    if (strcmp (name, chartypes[i].name) == 0)
-		return i;
-	    }
-	else
-	    {
-	    for (cp = chartypes[i].suffixes;  *cp != '\0';  cp += cplen + 1)
 		{
-		cplen = strlen (cp);
-		if (len >= cplen  &&  strcmp (&name[len - cplen], cp) == 0)
-		    return i;
+		if (deformatter != NULL)
+		    *deformatter =
+		      (strcmp (chartypes[i].deformatter, "tex") == 0);
+		return i;
+		}
+	    }
+	}
+    for (i = 0;  i < hashheader.nstrchartype;  i++)
+	{
+	for (cp = chartypes[i].suffixes;  *cp != '\0';  cp += cplen + 1)
+	    {
+	    cplen = strlen (cp);
+	    if (len >= cplen  &&  strcmp (&name[len - cplen], cp) == 0)
+		{
+		if (deformatter != NULL)
+		    *deformatter =
+		      (strcmp (chartypes[i].deformatter, "tex") == 0);
+		return i;
 		}
 	    }
 	}
@@ -1073,17 +1063,18 @@ findfiletype (name, searchnames)
  * The following routines are all dummies for the benefit of lint.
  */
 #ifdef lint
-TSTMASKBIT (mask, bit) MASKTYPE *mask; { return bit + (int) *mask; }
-CLRMASKBIT (mask, bit) MASKTYPE *mask; { bit += (int) *mask; }
-SETMASKBIT (mask, bit) MASKTYPE *mask; { bit += (int) *mask; }
-BITTOCHAR (bit) { return bit; }
-CHARTOBIT (ch) unsigned char ch; { return ch; }
-myupper (ch) ichar_t ch; { return (int) ch; }
-mylower (ch) ichar_t ch; { return (int) ch; }
-myspace (ch) ichar_t ch; { return (int) ch; }
-iswordch (ch) ichar_t ch; { return (int) ch; }
-isboundarych (ch) ichar_t ch; { return (int) ch; }
-isstringstart (ch) int ch; { return ch; }
-ichar_t mytolower (ch) ichar_t ch; { return (int) ch; }
-ichar_t mytoupper (ch) ichar_t ch; { return (int) ch; }
+int TSTMASKBIT (mask, bit) MASKTYPE * mask; int bit;
+    { return bit + (int) *mask; }
+void CLRMASKBIT (mask, bit) MASKTYPE * mask; int bit; { bit += (int) *mask; }
+void SETMASKBIT (mask, bit) MASKTYPE * mask; int bit; { bit += (int) *mask; }
+int BITTOCHAR (bit) int bit; { return bit; }
+int CHARTOBIT (ch) unsigned int ch; { return ch; }
+int myupper (ch) unsigned int ch; { return (int) ch; }
+int mylower (ch) unsigned int ch; { return (int) ch; }
+int myspace (ch) unsigned int ch; { return (int) ch; }
+int iswordch (ch) unsigned int ch; { return (int) ch; }
+int isboundarych (ch) unsigned int ch; { return (int) ch; }
+int isstringstart (ch) int ch; { return ch; }
+ichar_t mytolower (ch) unsigned int ch; { return (ichar_t) ch; }
+ichar_t mytoupper (ch) unsigned int ch; { return (ichar_t) ch; }
 #endif /* lint */
