@@ -1,7 +1,6 @@
-/* -*- Mode:Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: term.c,v 1.37 1991/12/09 00:40:48 geoff Exp $";
+    "$Id: term.c,v 1.46 1994/01/25 07:12:11 geoff Exp $";
 #endif
 
 /*
@@ -9,160 +8,79 @@ static char Rcs_Id[] =
  *
  * Pace Willisson, 1983
  *
- * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
+ * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 /*
  * $Log: term.c,v $
- * Revision 1.37  1991/12/09  00:40:48  geoff
- * Don't unlink the tempfile on exit if it has a null name.
+ * Revision 1.46  1994/01/25  07:12:11  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
- * Revision 1.36  91/09/04  18:00:57  geoff
- * Move the include of config.h.  Since it now contains the definition of
- * USG, it must precede any use of that option.
- * 
- * Revision 1.35  91/07/27  20:48:37  geoff
- * Don't obey the screen size from TIOCGWINSZ if it's zero or negative.
- * 
- * Revision 1.34  91/07/11  19:52:24  geoff
- * Remove the include of stdio.h, since ispell.h now does this.  Change
- * the termination code to put out the te sequence, rather than ti.
- * 
- * Revision 1.33  91/07/05  19:51:55  geoff
- * Fix some lint complaints, including getting rid of a couple of unused
- * variables.
- * 
- * Revision 1.32  91/07/03  18:21:09  geoff
- * Add code to support the "ti" and "te" termcap sequences, since the Sun
- * cmdtool (or is it shelltool) needs it to function properly with ispell.
- * 
- * Revision 1.31  90/12/31  00:59:29  geoff
- * Reformat to follow a consistent convention throughout ispell
- * 
- * Revision 1.30  90/04/26  22:44:25  geoff
- * If it is available, use the TIOCGWINSZ ioctl to determine the dimensions
- * of the terminal.  The enviroment variables LINES and COLUMNS can still
- * be used to override this ioctl.
- * 
- * Revision 1.29  90/04/17  15:35:07  geoff
- * Declare the signal-handling routines to be SIGNAL_TYPE
- * 
- * Revision 1.28  89/12/27  03:18:43  geoff
- * Move all messages to msgs.h so they can be reconfigured
- * 
- * Revision 1.27  89/12/26  23:26:32  geoff
- * Change the signal type to be definable (Israel Pinkas).
- * 
- * Revision 1.26  89/04/28  01:17:17  geoff
- * Change Header to Id;  nobody cares about my pathnames.
- * 
- * Revision 1.25  89/04/03  01:58:34  geoff
- * Fix a bunch of lint complaints.  Add code to let LINES and COLUMNS
- * override the screen size given by the termcap entry.  Rearrange some
- * code so that signal-catching and terminal setting is done only after
- * we are sure the termcap entry is ok.
- * 
- * Revision 1.24  89/02/27  02:23:48  geoff
- * Fix the interpretation of CONTEXTROUNDUP (it was inverted from what
- * the comments in config.X said).
- * 
- * Revision 1.23  89/02/22  23:16:43  geoff
- * Add support for calculating "contextsize" based on the terminal size.
- * 
- * Revision 1.22  89/01/06  00:14:47  geoff
- * Add support for MAX_SCREEN_SIZE (Keith Cantrell).
- * 
- * Revision 1.21  88/12/26  02:32:39  geoff
- * Add a copyright notice.
- * 
- * Revision 1.20  88/11/16  02:20:35  geoff
- * Make sure "tpgrp" is defined if TIOCGPGRP is defined but not TIOCPGRP
- * (a Locus-ism).
- * 
- * Revision 1.19  88/10/20  20:53:09  geoff
- * If a shell escape fails, don't flush stdio buffers on exit from the child.
- * (Ole Bjoern Hessen).
- * 
- * Revision 1.18  88/04/30  22:15:45  geoff
- * Fix some lint complaints, and get rid of some unused variables found
- * by lint.
- * 
- * Revision 1.17  88/03/27  01:05:19  geoff
- * Fix the #ifdefs to handle USG-like systems that nevertheless have job
- * control.
- * 
- * Revision 1.16  87/09/03  23:17:42  geoff
- * Integrate George Sipe's latest changes:  make ^Z spawn a shell on USG.
- * Add the shellescape() routine to handle simple shell commands faster.
- * Also clean up signal handling a bit.
- * 
- * Revision 1.15  87/09/03  17:32:29  geoff
- * Add ifdefs to make it compile on V7
- * 
- * Revision 1.14  87/09/03  17:09:09  geoff
- * Catch signals on USG systems, too (imt3b2!imtsft)
- * 
- * Revision 1.13  87/06/27  12:20:54  geoff
- * Get the sg termcap value for braindamaged terminals (Michael Wester).
- * 
- * Revision 1.12  87/06/07  14:31:12  geoff
- * Integrate Joel Shprentz's changes into the main branch
- * 
- * Revision 1.11  87/05/25  21:24:34  geoff
- * Integrate Mark Davies' changes, and improve some signal catching code.
- * 
- * Revision 1.10  87/04/21  23:29:20  geoff
- * Only catch signals if they aren't ignored;  this makes background stuff
- * work properly.
- * 
- * Revision 1.9  87/04/19  22:53:44  geoff
- * Swap the includes of ispell.h and config.h so config.h can affect ispell.h.
- * 
- * Revision 1.8  87/04/02  12:26:10  geoff
- * Make stdout buffered.
- * 
- * Revision 1.7  87/04/01  15:22:58  geoff
- * Integrate Joe Orost's V7/register changes into the main branch
- * 
- * Revision 1.6  87/03/22  23:21:44  geoff
- * Integrate Perry Smith's changes into the main branch
- * 
- * Revision 1.5  87/02/26  00:22:48  geoff
- * Integrate McQueer's and McMahon's enhancements into the main branch
- * 
- * Revision 1.4  87/01/19  00:25:19  geoff
- * Prohibit non-interactive use on USG systems too
- * 
- * Revision 1.3  87/01/19  00:07:00  geoff
- * Modify to work on USG systems.
- * 
- * Revision 1.2  87/01/17  13:12:11  geoff
- * Add RCS ID keywords
- * 
  */
 
 #include "config.h"
+#include "ispell.h"
+#include "proto.h"
+#include "msgs.h"
 #ifdef USG
 #include <termio.h>
 #else
 #include <sgtty.h>
 #endif
 #include <signal.h>
-#include "ispell.h"
-#include "msgs.h"
 
-extern char *	tgoto ();
+void		erase P ((void));
+void		move P ((int row, int col));
+void		inverse P ((void));
+void		normal P ((void));
+void		backup P ((void));
+static int	putch P ((int c));
+void		terminit P ((void));
+SIGNAL_TYPE	done P ((int signo));
+#ifdef SIGTSTP
+static SIGNAL_TYPE onstop P ((int signo));
+#endif /* SIGTSTP */
+void		stop P ((void));
+int		shellescape P ((char * buf));
+#ifdef USESH
+void		shescape P ((char * buf));
+#endif /* USESH */
 
-extern void	exit ();
-extern void	_exit ();
-extern char *	getenv ();
-
-int putch();
-
-erase ()
+void erase ()
     {
 
     if (cl)
@@ -177,22 +95,24 @@ erase ()
 	}
     }
 
-move (row, col)
+void move (row, col)
+    int		row;
+    int		col;
     {
     tputs (tgoto (cm, col, row), 100, putch);
     }
 
-inverse ()
+void inverse ()
     {
     tputs (so, 10, putch);
     }
 
-normal ()
+void normal ()
     {
     tputs (se, 10, putch);
     }
 
-backup ()
+void backup ()
     {
     if (BC)
 	tputs (BC, 1, putch);
@@ -200,20 +120,22 @@ backup ()
 	(void) putchar ('\b');
     }
 
-putch (c)
+static int putch (c)
+    int			c;
     {
-    (void) putchar (c);
+
+    return putchar (c);
     }
 
 #ifdef USG
-struct termio		sbuf;
-struct termio		osbuf;
+static struct termio	sbuf;
+static struct termio	osbuf;
 #else
-struct sgttyb		sbuf;
-struct sgttyb		osbuf;
+static struct sgttyb	sbuf;
+static struct sgttyb	osbuf;
 #ifdef TIOCSLTC
-struct ltchars		ltc;
-struct ltchars		oltc;
+static struct ltchars	ltc;
+static struct ltchars	oltc;
 #endif
 #endif
 static int		termchanged = 0;
@@ -225,7 +147,7 @@ static SIGNAL_TYPE	(*oldttou) ();
 static SIGNAL_TYPE	(*oldtstp) ();
 #endif
 
-terminit ()
+void terminit ()
     {
 #ifdef TIOCPGRP
     int			tpgrp;
@@ -234,9 +156,6 @@ terminit ()
     int			tpgrp;
 #endif
 #endif
-#ifdef SIGTSTP
-    SIGNAL_TYPE		onstop ();
-#endif
 #ifdef TIOCGWINSZ
     struct winsize	wsize;
 #endif /* TIOCGWINSZ */
@@ -258,11 +177,11 @@ terminit ()
     co = tgetnum ("co");
     li = tgetnum ("li");
 #ifdef TIOCGWINSZ
-    if (ioctl (0, TIOCGWINSZ, &wsize) >= 0)
+    if (ioctl (0, TIOCGWINSZ, (char *) &wsize) >= 0)
 	{
-	if (wsize.ws_col > 0)
+	if (wsize.ws_col != 0)
 	    co = wsize.ws_col;
-	if (wsize.ws_row > 0)
+	if (wsize.ws_row != 0)
 	    li = wsize.ws_row;
 	}
 #endif /* TIOCGWINSZ */
@@ -320,7 +239,7 @@ retry:
 	(void) fprintf (stderr, TERM_C_NO_BATCH);
 	exit (1);
 	}
-    (void) ioctl (0, TCGETA, &osbuf);
+    (void) ioctl (0, TCGETA, (char *) &osbuf);
     termchanged = 1;
 
     sbuf = osbuf;
@@ -329,7 +248,7 @@ retry:
     sbuf.c_iflag &= ~(INLCR | IGNCR | ICRNL);
     sbuf.c_cc[VMIN] = 1;
     sbuf.c_cc[VTIME] = 1;
-    (void) ioctl (0, TCSETAW, &sbuf);
+    (void) ioctl (0, TCSETAW, (char *) &sbuf);
 
     erasechar = osbuf.c_cc[VERASE];
     killchar = osbuf.c_cc[VKILL];
@@ -342,7 +261,7 @@ retry:
 #endif
 #endif
 #ifdef TIOCGPGRP
-    if (ioctl (0, TIOCGPGRP, &tpgrp) != 0)
+    if (ioctl (0, TIOCGPGRP, (char *) &tpgrp) != 0)
 	{
 	(void) fprintf (stderr, TERM_C_NO_BATCH);
 	exit (1);
@@ -364,16 +283,16 @@ retry:
 #endif
 
 #ifndef USG
-    (void) ioctl (0, TIOCGETP, &osbuf);
+    (void) ioctl (0, TIOCGETP, (char *) &osbuf);
 #ifdef TIOCGLTC
-    (void) ioctl (0, TIOCGLTC, &oltc);
+    (void) ioctl (0, TIOCGLTC, (char *) &oltc);
 #endif
     termchanged = 1;
 
     sbuf = osbuf;
     sbuf.sg_flags &= ~ECHO;
     sbuf.sg_flags |= TERM_MODE;
-    (void) ioctl (0, TIOCSETP, &sbuf);
+    (void) ioctl (0, TIOCSETP, (char *) &sbuf);
 
     erasechar = sbuf.sg_erase;
     killchar = sbuf.sg_kill;
@@ -381,7 +300,7 @@ retry:
 #ifdef TIOCSLTC
     ltc = oltc;
     ltc.t_suspc = -1;
-    (void) ioctl (0, TIOCSLTC, &ltc);
+    (void) ioctl (0, TIOCSLTC, (char *) &ltc);
 #endif
 
 #endif /* USG */
@@ -406,7 +325,9 @@ retry:
 	tputs (ti, 1, putch);
     }
 
-SIGNAL_TYPE done ()
+/* ARGSUSED */
+SIGNAL_TYPE done (signo)
+    int		signo;
     {
     if (tempfile[0] != '\0')
 	(void) unlink (tempfile);
@@ -415,11 +336,11 @@ SIGNAL_TYPE done ()
 	if (te)
 	    tputs (te, 1, putch);
 #ifdef USG
-	(void) ioctl (0, TCSETAW, &osbuf);
+	(void) ioctl (0, TCSETAW, (char *) &osbuf);
 #else
-	(void) ioctl (0, TIOCSETP, &osbuf);
+	(void) ioctl (0, TIOCSETP, (char *) &osbuf);
 #ifdef TIOCSLTC
-	(void) ioctl (0, TIOCSLTC, &oltc);
+	(void) ioctl (0, TIOCSLTC, (char *) &oltc);
 #endif
 #endif
 	}
@@ -427,15 +348,15 @@ SIGNAL_TYPE done ()
     }
 
 #ifdef SIGTSTP
-SIGNAL_TYPE onstop (signo)
+static SIGNAL_TYPE onstop (signo)
     int		signo;
     {
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &osbuf);
+    (void) ioctl (0, TCSETAW, (char *) &osbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &osbuf);
+    (void) ioctl (0, TIOCSETP, (char *) &osbuf);
 #ifdef TIOCSLTC
-    (void) ioctl (0, TIOCSLTC, &oltc);
+    (void) ioctl (0, TIOCSLTC, (char *) &oltc);
 #endif
 #endif
     (void) signal (signo, SIG_DFL);
@@ -446,17 +367,17 @@ SIGNAL_TYPE onstop (signo)
     /* stop here until continued */
     (void) signal (signo, onstop);
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &sbuf);
+    (void) ioctl (0, TCSETAW, (char *) &sbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &sbuf);
+    (void) ioctl (0, TIOCSETP, (char *) &sbuf);
 #ifdef TIOCSLTC
-    (void) ioctl (0, TIOCSLTC, &ltc);
+    (void) ioctl (0, TIOCSLTC, (char *) &ltc);
 #endif
 #endif
     }
 #endif
 
-stop ()
+void stop ()
     {
 #ifdef SIGTSTP
     onstop (SIGTSTP);
@@ -475,6 +396,13 @@ stop ()
 ** command's return status.  Returns zero if command was not found.
 ** Doesn't use a shell.
 */
+#ifndef USESH
+#define NEED_SHELLESCAPE
+#endif /* USESH */
+#ifndef REGEX_LOOKUP
+#define NEED_SHELLESCAPE
+#endif /* REGEX_LOOKUP */
+#ifdef NEED_SHELLESCAPE
 int shellescape	(buf)
     char *	buf;
     {
@@ -499,10 +427,10 @@ int shellescape	(buf)
     argv[i] = NULL;
 
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &osbuf);
+    (void) ioctl (0, TCSETAW, (char *) &osbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &osbuf);
-    (void) ioctl (0, TIOCSLTC, &oltc);
+    (void) ioctl (0, TIOCSETP, (char *) &osbuf);
+    (void) ioctl (0, TIOCSLTC, (char *) &oltc);
 #endif
     (void) signal (SIGINT, oldint);
     (void) signal (SIGTERM, oldterm);
@@ -513,7 +441,7 @@ int shellescape	(buf)
 #endif
     if ((i = fork ()) == 0)
 	{
-	(void) execvp (argv[0], argv);
+	(void) execvp (argv[0], (char **) argv);
 	_exit (123);		/* Command not found */
 	}
     else if (i > 0)
@@ -543,29 +471,30 @@ int shellescape	(buf)
 #endif
 
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &sbuf);
+    (void) ioctl (0, TCSETAW, (char *) &sbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &sbuf);
-    (void) ioctl (0, TIOCSLTC, &ltc);
+    (void) ioctl (0, TIOCSETP, (char *) &sbuf);
+    (void) ioctl (0, TIOCSLTC, (char *) &ltc);
 #endif
     if (termstat)
 	{
 	(void) printf (TERM_C_TYPE_SPACE);
 	(void) fflush (stdout);
 #ifdef COMMANDFORSPACE
-	i = getchar ();
+	i = GETKEYSTROKE ();
 	if (i != ' ' && i != '\n' && i != '\r')
-	    ungetc (i, stdin);
+	    (void) ungetc (i, stdin);
 #else
-	while (getchar () != ' ')
+	while (GETKEYSTROKE () != ' ')
 	    ;
 #endif
 	}
     return (termstat);
     }
+#endif /* NEED_SHELLESCAPE */
 
 #ifdef	USESH
-shescape (buf)
+void shescape (buf)
     char *	buf;
     {
 #ifdef COMMANDFORSPACE
@@ -573,11 +502,11 @@ shescape (buf)
 #endif
 
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &osbuf);
+    (void) ioctl (0, TCSETAW, (char *) &osbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &osbuf);
+    (void) ioctl (0, TIOCSETP, (char *) &osbuf);
 #ifdef TIOCSLTC
-    (void) ioctl (0, TIOCSLTC, &oltc);
+    (void) ioctl (0, TIOCSLTC, (char *) &oltc);
 #endif
 #endif
     (void) signal (SIGINT, oldint);
@@ -605,21 +534,21 @@ shescape (buf)
 #endif
 
 #ifdef USG
-    (void) ioctl (0, TCSETAW, &sbuf);
+    (void) ioctl (0, TCSETAW, (char *) &sbuf);
 #else
-    (void) ioctl (0, TIOCSETP, &sbuf);
+    (void) ioctl (0, TIOCSETP, (char *) &sbuf);
 #ifdef TIOCSLTC
-    (void) ioctl (0, TIOCSLTC, &ltc);
+    (void) ioctl (0, TIOCSLTC, (char *) &ltc);
 #endif
 #endif
     (void) printf (TERM_C_TYPE_SPACE);
     (void) fflush (stdout);
 #ifdef COMMANDFORSPACE
-    ch = getchar ();
+    ch = GETKEYSTROKE ();
     if (ch != ' '  &&  ch != '\n'  &&  ch != '\r')
-	ungetc (ch, stdin);
+	(void) ungetc (ch, stdin);
 #else
-    while (getchar () != ' ')
+    while (GETKEYSTROKE () != ' ')
 	;
 #endif
     }
