@@ -1,81 +1,44 @@
-/* -*- Mode:Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: tgood.c,v 1.18 91/09/11 23:23:00 geoff Exp $";
+    "$Id: tgood.c,v 1.30 1994/05/24 06:23:08 geoff Exp $";
 #endif
 
 /*
- * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
- * Permission for non-profit use is hereby granted.
- * All other rights reserved.
- * See "version.h" for a more complete copyright notice.
- */
-
-/*
- * $Log:	tgood.c,v $
- * Revision 1.18  91/09/11  23:23:00  geoff
- * Make sure that pure-deletion suffixes operate correctly, by adding the
- * necessary null byte after the strip portion.
- * 
- * Revision 1.17  91/07/11  19:52:27  geoff
- * Remove the include of stdio.h, since ispell.h now does this.
- * 
- * Revision 1.16  91/05/27  21:48:12  geoff
- * When checking affixes, remember to check for the case of affixes that
- * strip nothing and only add characters.
- * 
- * Revision 1.15  90/12/31  01:04:16  geoff
- * Add a text-mode line so emacs will edit consistently.
- * 
- * Revision 1.14  90/04/26  22:44:28  geoff
- * Add the canonicalize parameter to calls to ichartosstr.
- * 
- * Revision 1.13  89/06/09  15:56:32  geoff
- * Add support for the internal "character" type, ichar_t.
- * 
- * Revision 1.12  89/04/27  23:34:27  geoff
- * In suf_list_chk, call setfindlast just before the condition check, because
- * the word changes on each pass through the loop.  Also call findlastchar
- * with the correct parameters.
- * 
- * Revision 1.11  89/04/03  01:58:53  geoff
- * Fix a bunch of lint complaints.  Add (partial) support for string
- * characters.
- * 
- * Revision 1.10  88/12/26  02:33:04  geoff
- * Update the copyright notice.
- * 
- * Revision 1.9  88/11/25  19:54:28  geoff
- * Fix some inconsistencies in the formatting of curly braces.
- * 
- * Revision 1.8  88/04/30  22:16:04  geoff
- * Fix some lint complaints.
- * 
- * Revision 1.7  88/02/20  23:14:44  geoff
- * Support the new capitalization handling.  Add the "allhits" parameter to
- * chk_aff and its descendents.
- * 
- * Revision 1.6  87/10/02  22:38:11  geoff
- * Modify expansion routines to exactly duplicate the capitalization
- * rules implemented by ins_cap and cap_ok.
- * 
- * Revision 1.5  87/09/30  23:31:48  geoff
- * Move some globals to ispell.h.  Add support for ignoreflagbits.
- * Rationalize some array sizes.  Get rid of the checkbit calls and
- * replace them with a macro call.
- * 
- * Revision 1.4  87/09/26  15:41:25  geoff
- * Make the sizes of the word buffers consistent.
- * 
- * Revision 1.3  87/09/14  22:39:19  geoff
- * Add copyright comments
- * 
- * Revision 1.2  87/09/09  00:19:43  geoff
- * Replace a lot of #ifdef's on NO8BIT with use of SET_SIZE and NOPARITY.
- * 
- * Revision 1.1  87/07/20  22:56:14  geoff
- * Initial revision
- * 
+ * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All modifications to the source code must be clearly marked as
+ *    such.  Binary redistributions based on modified source code
+ *    must be clearly marked as modified versions in the documentation
+ *    and/or other materials provided with the distribution.
+ * 4. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgment:
+ *      This product includes software developed by Geoff Kuenning and
+ *      other unpaid contributors.
+ * 5. The name of Geoff Kuenning may not be used to endorse or promote
+ *    products derived from this software without specific prior
+ *    written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 /*
@@ -84,26 +47,64 @@ static char Rcs_Id[] =
  * Geoff Kuenning, July 1987
  */
 
+/*
+ * $Log: tgood.c,v $
+ * Revision 1.30  1994/05/24  06:23:08  geoff
+ * Don't create a hit if "allhits" is clear and capitalization
+ * mismatches.  This cures a bug where a word could be in the dictionary
+ * and yet not found.
+ *
+ * Revision 1.29  1994/05/17  06:44:21  geoff
+ * Add support for controlled compound formation and the COMPOUNDONLY
+ * option to affix flags.
+ *
+ * Revision 1.28  1994/01/25  07:12:13  geoff
+ * Get rid of all old RCS log lines in preparation for the 3.1 release.
+ *
+ */
+
 #include <ctype.h>
 #include "config.h"
 #include "ispell.h"
+#include "proto.h"
 
-extern struct dent *	lookup ();	/* Look up a dictionary entry */
-
-extern char *		strcpy ();
+void		chk_aff P ((ichar_t * word, ichar_t * ucword, int len,
+		  int ignoreflagbits, int allhits, int pfxopts, int sfxopts));
+static void	pfx_list_chk P ((ichar_t * word, ichar_t * ucword,
+		  int len, int optflags, int sfxopts, struct flagptr * ind,
+		  int ignoreflagbits, int allhits));
+static void	chk_suf P ((ichar_t * word, ichar_t * ucword, int len,
+		  int optflags, struct flagent * pfxent, int ignoreflagbits,
+		  int allhits));
+static void	suf_list_chk P ((ichar_t * word, ichar_t * ucword, int len,
+		  struct flagptr * ind, int optflags, struct flagent * pfxent,
+		  int ignoreflagbits, int allhits));
+int		expand_pre P ((char * croot, ichar_t * rootword,
+		  MASKTYPE mask[], int option, char * extra));
+static int	pr_pre_expansion P ((char * croot, ichar_t * rootword,
+		  struct flagent * flent, MASKTYPE mask[], int option,
+		  char * extra));
+int		expand_suf P ((char * croot, ichar_t * rootword,
+		  MASKTYPE mask[], int optflags, int option, char * extra));
+static int	pr_suf_expansion P ((char * croot, ichar_t * rootword,
+		  struct flagent * flent, int option, char * extra));
+static void	forcelc P ((ichar_t * dst, int len));
 
 /* Check possible affixes */
-chk_aff (word, ucword, len, ignoreflagbits, allhits)
+void chk_aff (word, ucword, len, ignoreflagbits, allhits, pfxopts, sfxopts)
     ichar_t *		word;		/* Word to be checked */
     ichar_t *		ucword;		/* Upper-case-only copy of word */
     int			len;		/* The length of word/ucword */
     int			ignoreflagbits;	/* Ignore whether affix is legal */
     int			allhits;	/* Keep going after first hit */
+    int			pfxopts;	/* Options to apply to prefixes */
+    int			sfxopts;	/* Options to apply to suffixes */
     {
     register ichar_t *	cp;		/* Pointer to char to index on */
     struct flagptr *	ind;		/* Flag index table to test */
 
-    pfx_list_chk (word, ucword, len, &pflagindex[0], ignoreflagbits, allhits);
+    pfx_list_chk (word, ucword, len, pfxopts, sfxopts, &pflagindex[0],
+      ignoreflagbits, allhits);
     cp = ucword;
     ind = &pflagindex[*cp++];
     while (ind->numents == 0  &&  ind->pu.fp != NULL)
@@ -112,25 +113,29 @@ chk_aff (word, ucword, len, ignoreflagbits, allhits)
 	    return;
 	if (ind->pu.fp[0].numents)
 	    {
-	    pfx_list_chk (word, ucword, len, &ind->pu.fp[0],
+	    pfx_list_chk (word, ucword, len, pfxopts, sfxopts, &ind->pu.fp[0],
 	      ignoreflagbits, allhits);
 	    if (numhits  &&  !allhits  &&  !cflag  &&  !ignoreflagbits)
 		return;
 	    }
 	ind = &ind->pu.fp[*cp++];
 	}
-    pfx_list_chk (word, ucword, len, ind, ignoreflagbits, allhits);
+    pfx_list_chk (word, ucword, len, pfxopts, sfxopts, ind, ignoreflagbits,
+      allhits);
     if (numhits  &&  !allhits  &&  !cflag  &&  !ignoreflagbits)
 	return;
-    chk_suf (word, ucword, len, 0, (struct flagent *) NULL,
+    chk_suf (word, ucword, len, sfxopts, (struct flagent *) NULL,
       ignoreflagbits, allhits);
     }
 
 /* Check some prefix flags */
-pfx_list_chk (word, ucword, len, ind, ignoreflagbits, allhits)
+static void pfx_list_chk (word, ucword, len, optflags, sfxopts, ind,
+  ignoreflagbits, allhits)
     ichar_t *		word;		/* Word to be checked */
     ichar_t *		ucword;		/* Upper-case-only word */
     int			len;		/* The length of ucword */
+    int			optflags;	/* Options to apply */
+    int			sfxopts;	/* Options to apply to suffixes */
     struct flagptr *	ind;		/* Flag index table */
     int			ignoreflagbits;	/* Ignore whether affix is legal */
     int			allhits;	/* Keep going after first hit */
@@ -150,12 +155,28 @@ pfx_list_chk (word, ucword, len, ind, ignoreflagbits, allhits)
       entcount > 0;
       flent++, entcount--)
 	{
+	/*
+	 * If this is a compound-only affix, ignore it unless we're
+	 * looking fotr that specific thing.
+	 */
+	if ((flent->flagflags & FF_COMPOUNDONLY) != 0
+	  &&  (optflags & FF_COMPOUNDONLY) == 0)
+	    continue;
+	/*
+	 * In COMPOUND_CONTROLLED mode, the FF_COMPOUNDONLY bit must
+	 * match exactly.
+	 */
+	if (compoundflag == COMPOUND_CONTROLLED
+	  &&  ((flent->flagflags ^ optflags) & FF_COMPOUNDONLY) != 0)
+	    continue;
 	/*
 	 * See if the prefix matches.
 	 */
-	tlen = len - flent->affl + flent->stripl;
-	if (tlen > 0  &&  icharncmp (flent->affix, ucword, flent->affl) == 0
-	  &&  tlen >= flent->numconds)
+	tlen = len - flent->affl;
+	if (tlen > 0
+	  &&  (flent->affl == 0
+	    ||  icharncmp (flent->affix, ucword, flent->affl) == 0)
+	  &&  tlen + flent->stripl >= flent->numconds)
 	    {
 	    /*
 	     * The prefix matches.  Remove it, replace it by the "strip"
@@ -176,6 +197,7 @@ pfx_list_chk (word, ucword, len, ind, ignoreflagbits, allhits)
 		 * The conditions match.  See if the word is in the
 		 * dictionary.
 		 */
+		tlen += flent->stripl;
 		if (cflag)
 		    flagpr (tword, BITTOCHAR (flent->flagbit), flent->stripl,
 		      flent->affl, -1, 0);
@@ -215,33 +237,42 @@ pfx_list_chk (word, ucword, len, ind, ignoreflagbits, allhits)
 			numhits++;
 			}
 		    if (!allhits)
+			{
+#ifndef NO_CAPITALIZATION_SUPPORT
+			if (cap_ok (word, &hits[0], len))
+			    return;
+			numhits = 0;
+#else /* NO_CAPITALIZATION_SUPPORT */
 			return;
+#endif /* NO_CAPITALIZATION_SUPPORT */
+			}
 		    }
 		/*
 		 * Handle cross-products.
 		 */
 		if (flent->flagflags & FF_CROSSPRODUCT)
-		    chk_suf (word, tword, tlen, 1, flent,
-		      ignoreflagbits, allhits);
+		    chk_suf (word, tword, tlen, sfxopts | FF_CROSSPRODUCT,
+		      flent, ignoreflagbits, allhits);
 		}
 	    }
 	}
     }
 
 /* Check possible suffixes */
-chk_suf (word, ucword, len, crossonly, pfxent, ignoreflagbits, allhits)
+static void chk_suf (word, ucword, len, optflags, pfxent, ignoreflagbits,
+  allhits)
     ichar_t *		word;		/* Word to be checked */
     ichar_t *		ucword;		/* Upper-case-only word */
     int			len;		/* The length of ucword */
-    int			crossonly;	/* NZ to do only cross-products */
-    struct flagent *	pfxent;		/* Prefix flag entry if crossonly */
+    int			optflags;	/* Affix option flags */
+    struct flagent *	pfxent;		/* Prefix flag entry if cross-prod */
     int			ignoreflagbits;	/* Ignore whether affix is legal */
     int			allhits;	/* Keep going after first hit */
     {
     register ichar_t *	cp;		/* Pointer to char to index on */
     struct flagptr *	ind;		/* Flag index table to test */
 
-    suf_list_chk (word, ucword, len, &sflagindex[0], crossonly, pfxent,
+    suf_list_chk (word, ucword, len, &sflagindex[0], optflags, pfxent,
       ignoreflagbits, allhits);
     cp = ucword + len - 1;
     ind = &sflagindex[*cp];
@@ -252,23 +283,23 @@ chk_suf (word, ucword, len, crossonly, pfxent, ignoreflagbits, allhits)
 	if (ind->pu.fp[0].numents)
 	    {
 	    suf_list_chk (word, ucword, len, &ind->pu.fp[0],
-	      crossonly, pfxent, ignoreflagbits, allhits);
+	      optflags, pfxent, ignoreflagbits, allhits);
 	    if (numhits != 0  &&  !allhits  &&  !cflag  &&  !ignoreflagbits)
 		return;
 	    }
 	ind = &ind->pu.fp[*--cp];
 	}
-    suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
+    suf_list_chk (word, ucword, len, ind, optflags, pfxent,
       ignoreflagbits, allhits);
     }
     
-suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
+static void suf_list_chk (word, ucword, len, ind, optflags, pfxent,
   ignoreflagbits, allhits)
     ichar_t *		word;		/* Word to be checked */
     ichar_t *		ucword;		/* Upper-case-only word */
     int			len;		/* The length of ucword */
     struct flagptr *	ind;		/* Flag index table */
-    int			crossonly;	/* NZ to do only cross-products */
+    int			optflags;	/* Affix option flags */
     struct flagent *	pfxent;		/* Prefix flag entry if crossonly */
     int			ignoreflagbits;	/* Ignore whether affix is legal */
     int			allhits;	/* Keep going after first hit */
@@ -289,14 +320,28 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
       entcount > 0;
       flent++, entcount--)
 	{
-	if (crossonly  &&  (flent->flagflags & FF_CROSSPRODUCT) == 0)
+	if ((optflags & FF_CROSSPRODUCT) != 0
+	  &&  (flent->flagflags & FF_CROSSPRODUCT) == 0)
+	    continue;
+	/*
+	 * If this is a compound-only affix, ignore it unless we're
+	 * looking fotr that specific thing.
+	 */
+	if ((flent->flagflags & FF_COMPOUNDONLY) != 0
+	  &&  (optflags & FF_COMPOUNDONLY) == 0)
+	    continue;
+	/*
+	 * In COMPOUND_CONTROLLED mode, the FF_COMPOUNDONLY bit must
+	 * match exactly.
+	 */
+	if (compoundflag == COMPOUND_CONTROLLED
+	  &&  ((flent->flagflags ^ optflags) & FF_COMPOUNDONLY) != 0)
 	    continue;
 	/*
 	 * See if the suffix matches.
 	 */
 	tlen = len - flent->affl;
 	if (tlen > 0
-	  &&  tlen + flent->stripl > 1
 	  &&  (flent->affl == 0
 	    ||  icharcmp (flent->affix, ucword + tlen) == 0)
 	  &&  tlen + flent->stripl >= flent->numconds)
@@ -328,7 +373,7 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 		 */
 		if (cflag)
 		    {
-		    if (crossonly)
+		    if (optflags & FF_CROSSPRODUCT)
 			flagpr (tword, BITTOCHAR (pfxent->flagbit),
 			  pfxent->stripl, pfxent->affl,
 			  BITTOCHAR (flent->flagbit), flent->affl);
@@ -341,7 +386,8 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 		    if ((dent = lookup (tword, 1)) != NULL)
 			{
 			cp = tword2;
-			if (crossonly  &&  pfxent->affl != 0)
+			if ((optflags & FF_CROSSPRODUCT)
+			  &&  pfxent->affl != 0)
 			    {
 			    (void) icharcpy (cp, pfxent->affix);
 			    cp += pfxent->affl;
@@ -350,7 +396,8 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 			preadd = cp - tword2;
 			(void) icharcpy (cp, tword);
 			cp += tlen;
-			if (crossonly  &&  pfxent->stripl != 0)
+			if ((optflags & FF_CROSSPRODUCT)
+			  &&  pfxent->stripl != 0)
 			    {
 			    *cp++ = '-';
 			    (void) icharcpy (cp, pfxent->strip);
@@ -369,14 +416,16 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 			    cp += flent->affl;
 			    }
 			(void) ins_root_cap (tword2, word,
-			  crossonly ? pfxent->stripl : 0, preadd,
+			  (optflags & FF_CROSSPRODUCT) ? pfxent->stripl : 0,
+			  preadd,
 			  flent->stripl, (cp - tword2) - tlen - preadd,
 			  dent, pfxent, flent);
 			}
 		    }
 		else if ((dent = lookup (tword, 1)) != NULL
 		  &&  TSTMASKBIT (dent->mask, flent->flagbit)
-		  &&  (!crossonly || TSTMASKBIT (dent->mask, pfxent->flagbit)))
+		  &&  ((optflags & FF_CROSSPRODUCT) == 0
+		    || TSTMASKBIT (dent->mask, pfxent->flagbit)))
 		    {
 		    if (numhits < MAX_HITS)
 			{
@@ -386,7 +435,15 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 			numhits++;
 			}
 		    if (!allhits)
+			{
+#ifndef NO_CAPITALIZATION_SUPPORT
+			if (cap_ok (word, &hits[0], len))
+			    return;
+			numhits = 0;
+#else /* NO_CAPITALIZATION_SUPPORT */
 			return;
+#endif /* NO_CAPITALIZATION_SUPPORT */
+			}
 		    }
 		}
 	    }
@@ -396,42 +453,54 @@ suf_list_chk (word, ucword, len, ind, crossonly, pfxent,
 /*
  * Expand a dictionary prefix entry
  */
-expand_pre (rootword, mask)
+int expand_pre (croot, rootword, mask, option, extra)
+    char *			croot;		/* Char version of rootword */
     ichar_t *			rootword;	/* Root word to expand */
     register MASKTYPE		mask[];		/* Mask bits to expand on */
+    int				option;		/* Option, see expandmode */
+    char *			extra;		/* Extra info to add to line */
     {
     int				entcount;	/* No. of entries to process */
+    int				explength;	/* Length of expansions */
     register struct flagent *
 				flent;		/* Current table entry */
 
-    for (flent = pflaglist, entcount = numpflags;
+    for (flent = pflaglist, entcount = numpflags, explength = 0;
       entcount > 0;
       flent++, entcount--)
 	{
 	if (TSTMASKBIT (mask, flent->flagbit))
-	    pr_pre_expansion (rootword, flent, mask);
+	    explength +=
+	      pr_pre_expansion (croot, rootword, flent, mask, option, extra);
 	}
+    return explength;
     }
 
-pr_pre_expansion (rootword, flent, mask) /* Print a prefix expansion */
-    register ichar_t *		rootword; /* Root word to expand */
-    register struct flagent *	flent;	/* Current table entry */
+/* Print a prefix expansion */
+static int pr_pre_expansion (croot, rootword, flent, mask, option, extra)
+    char *			croot;		/* Char version of rootword */
+    register ichar_t *		rootword;	/* Root word to expand */
+    register struct flagent *	flent;		/* Current table entry */
     MASKTYPE			mask[];		/* Mask bits to expand on */
+    int				option;		/* Option, see	expandmode */
+    char *			extra;		/* Extra info to add to line */
     {
-    int				cond;	/* Current condition number */
-    register ichar_t *		nextc;	/* Next case choice */
-    int				tlen;	/* Length of tword */
-    ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Tmp cpy */
+    int				cond;		/* Current condition number */
+    register ichar_t *		nextc;		/* Next case choice */
+    int				tlen;		/* Length of tword */
+    ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Temp */
 
     tlen = icharlen (rootword);
     if (flent->numconds > tlen)
-	return;
-    if (tlen - flent->stripl + flent->affl <= 0)
-	return;
+	return 0;
+    tlen -= flent->stripl;
+    if (tlen <= 0)
+	return 0;
+    tlen += flent->affl;
     for (cond = 0, nextc = rootword;  cond < flent->numconds;  cond++)
 	{
 	if ((flent->conds[mytoupper (*nextc++)] & (1 << cond)) == 0)
-	    return;
+	    return 0;
 	}
     /*
      * The conditions are satisfied.  Copy the word, add the prefix,
@@ -454,78 +523,104 @@ pr_pre_expansion (rootword, flent, mask) /* Print a prefix expansion */
     (void) icharcpy (nextc, rootword + flent->stripl);
     if (myupper (rootword[0]))
 	{
-	/* We must distinguish followcase from capitalized */
-	for (nextc = rootword + 1;  *nextc;  )
+	/* We must distinguish followcase from capitalized and all-upper */
+	for (nextc = rootword + 1;  *nextc;  nextc++)
 	    {
-	    if (!myupper (*nextc++))
+	    if (!myupper (*nextc))
 		break;
 	    }
 	if (*nextc)
 	    {
-	    /* It's a followcase or all-upper word.  We don't care which. */
-	    if (!myupper (tword[flent->affl]));
-		forcelc (tword, flent->affl);
-	    }
-	else
-	    {
-	    /* It's capitalized */
-	    forcelc (tword + 1, tlen + flent->affl - flent->stripl);
+	    /* It's a followcase or capitalized word.  Figure out which. */
+	    for (  ;  *nextc;  nextc++)
+		{
+		if (myupper (*nextc))
+		    break;
+		}
+	    if (*nextc)
+		{
+		/* It's followcase. */
+		if (!myupper (tword[flent->affl]))
+		    forcelc (tword, flent->affl);
+		}
+	    else
+		{
+		/* It's capitalized */
+		forcelc (tword + 1, tlen - 1);
+		}
 	    }
 	}
     else
 	{
 	/* Followcase or all-lower, we don't care which */
-	if (!myupper (*nextc));
+	if (!myupper (*nextc))
 	    forcelc (tword, flent->affl);
 	}
-    (void) printf (" %s", ichartosstr (tword, 1));
+    if (option == 3)
+	(void) printf ("\n%s", croot);
+    if (option != 4)
+	(void) printf (" %s%s", ichartosstr (tword, 1), extra);
     if (flent->flagflags & FF_CROSSPRODUCT)
-	expand_suf (tword, mask, 1);
+	return tlen
+	  + expand_suf (croot, tword, mask, FF_CROSSPRODUCT, option, extra);
+    else
+	return tlen;
     }
 
 /*
  * Expand a dictionary suffix entry
  */
-expand_suf (rootword, mask, crossonly)
+int expand_suf (croot, rootword, mask, optflags, option, extra)
+    char *			croot;		/* Char version of rootword */
     ichar_t *			rootword;	/* Root word to expand */
     register MASKTYPE		mask[];		/* Mask bits to expand on */
-    int				crossonly;	/* NZ for cross-products only */
+    int				optflags;	/* Affix option flags */
+    int				option;		/* Option, see expandmode */
+    char *			extra;		/* Extra info to add to line */
     {
     int				entcount;	/* No. of entries to process */
+    int				explength;	/* Length of expansions */
     register struct flagent *
 				flent;		/* Current table entry */
 
-    for (flent = sflaglist, entcount = numsflags;
+    for (flent = sflaglist, entcount = numsflags, explength = 0;
       entcount > 0;
       flent++, entcount--)
 	{
 	if (TSTMASKBIT (mask, flent->flagbit))
 	    {
-	    if (!crossonly  ||  (flent->flagflags & FF_CROSSPRODUCT))
-		pr_suf_expansion (rootword, flent);
+	    if ((optflags & FF_CROSSPRODUCT) == 0
+	      ||  (flent->flagflags & FF_CROSSPRODUCT))
+		explength +=
+		  pr_suf_expansion (croot, rootword, flent, option, extra);
 	    }
 	}
+    return explength;
     }
 
-pr_suf_expansion (rootword, flent)	/* Print a suffix expansion */
-    register ichar_t *		rootword; /* Root word to expand */
-    register struct flagent *	flent;	/* Current table entry */
+/* Print a suffix expansion */
+static int pr_suf_expansion (croot, rootword, flent, option, extra)
+    char *			croot;		/* Char version of rootword */
+    register ichar_t *		rootword;	/* Root word to expand */
+    register struct flagent *	flent;		/* Current table entry */
+    int				option;		/* Option, see expandmode */
+    char *			extra;		/* Extra info to add to line */
     {
-    int				cond;	/* Current condition number */
-    register ichar_t *		nextc;	/* Next case choice */
-    int				tlen;	/* Length of tword */
-    ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Tmp cpy */
+    int				cond;		/* Current condition number */
+    register ichar_t *		nextc;		/* Next case choice */
+    int				tlen;		/* Length of tword */
+    ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Temp */
 
     tlen = icharlen (rootword);
     cond = flent->numconds;
     if (cond > tlen)
-	return;
-    if (tlen - flent->stripl + flent->affl <= 0)
-	return;
+	return 0;
+    if (tlen - flent->stripl <= 0)
+	return 0;
     for (nextc = rootword + tlen;  --cond >= 0;  )
 	{
 	if ((flent->conds[mytoupper (*--nextc)] & (1 << cond)) == 0)
-		return;
+	    return 0;
 	}
     /*
      * The conditions are satisfied.  Copy the word, add the suffix,
@@ -542,10 +637,14 @@ pr_suf_expansion (rootword, flent)	/* Print a suffix expansion */
 	}
     else
 	*nextc = 0;
-    (void) printf (" %s", ichartosstr (tword, 1));
+    if (option == 3)
+	(void) printf ("\n%s", croot);
+    if (option != 4)
+	(void) printf (" %s%s", ichartosstr (tword, 1), extra);
+    return tlen + flent->affl - flent->stripl;
     }
 
-forcelc (dst, len)				/* Force to lowercase */
+static void forcelc (dst, len)			/* Force to lowercase */
     register ichar_t *		dst;		/* Destination to modify */
     register int		len;		/* Length to copy */
     {
